# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.appplatform import AppPlatformManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAppPlatformManagementDeploymentsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AppPlatformManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_get(self, resource_group):
        response = self.client.deployments.get(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            deployment_name="str",
            api_version="2023-12-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_create_or_update(self, resource_group):
        response = self.client.deployments.begin_create_or_update(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            deployment_name="str",
            deployment_resource={
                "id": "str",
                "name": "str",
                "properties": {
                    "active": bool,
                    "deploymentSettings": {
                        "addonConfigs": {"str": {}},
                        "apms": [{"resourceId": "str"}],
                        "containerProbeSettings": {"disableProbe": bool},
                        "environmentVariables": {"str": "str"},
                        "livenessProbe": {
                            "disableProbe": False,
                            "failureThreshold": 0,
                            "initialDelaySeconds": 0,
                            "periodSeconds": 0,
                            "probeAction": "probe_action",
                            "successThreshold": 0,
                            "timeoutSeconds": 0,
                        },
                        "readinessProbe": {
                            "disableProbe": False,
                            "failureThreshold": 0,
                            "initialDelaySeconds": 0,
                            "periodSeconds": 0,
                            "probeAction": "probe_action",
                            "successThreshold": 0,
                            "timeoutSeconds": 0,
                        },
                        "resourceRequests": {"cpu": "str", "memory": "str"},
                        "startupProbe": {
                            "disableProbe": False,
                            "failureThreshold": 0,
                            "initialDelaySeconds": 0,
                            "periodSeconds": 0,
                            "probeAction": "probe_action",
                            "successThreshold": 0,
                            "timeoutSeconds": 0,
                        },
                        "terminationGracePeriodSeconds": 90,
                    },
                    "instances": [
                        {
                            "discoveryStatus": "str",
                            "name": "str",
                            "reason": "str",
                            "startTime": "str",
                            "status": "str",
                            "zone": "str",
                        }
                    ],
                    "provisioningState": "str",
                    "source": "user_source_info",
                    "status": "str",
                },
                "sku": {"capacity": 0, "name": "S0", "tier": "Standard"},
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "type": "str",
            },
            api_version="2023-12-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_delete(self, resource_group):
        response = self.client.deployments.begin_delete(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            deployment_name="str",
            api_version="2023-12-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_update(self, resource_group):
        response = self.client.deployments.begin_update(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            deployment_name="str",
            deployment_resource={
                "id": "str",
                "name": "str",
                "properties": {
                    "active": bool,
                    "deploymentSettings": {
                        "addonConfigs": {"str": {}},
                        "apms": [{"resourceId": "str"}],
                        "containerProbeSettings": {"disableProbe": bool},
                        "environmentVariables": {"str": "str"},
                        "livenessProbe": {
                            "disableProbe": False,
                            "failureThreshold": 0,
                            "initialDelaySeconds": 0,
                            "periodSeconds": 0,
                            "probeAction": "probe_action",
                            "successThreshold": 0,
                            "timeoutSeconds": 0,
                        },
                        "readinessProbe": {
                            "disableProbe": False,
                            "failureThreshold": 0,
                            "initialDelaySeconds": 0,
                            "periodSeconds": 0,
                            "probeAction": "probe_action",
                            "successThreshold": 0,
                            "timeoutSeconds": 0,
                        },
                        "resourceRequests": {"cpu": "str", "memory": "str"},
                        "startupProbe": {
                            "disableProbe": False,
                            "failureThreshold": 0,
                            "initialDelaySeconds": 0,
                            "periodSeconds": 0,
                            "probeAction": "probe_action",
                            "successThreshold": 0,
                            "timeoutSeconds": 0,
                        },
                        "terminationGracePeriodSeconds": 90,
                    },
                    "instances": [
                        {
                            "discoveryStatus": "str",
                            "name": "str",
                            "reason": "str",
                            "startTime": "str",
                            "status": "str",
                            "zone": "str",
                        }
                    ],
                    "provisioningState": "str",
                    "source": "user_source_info",
                    "status": "str",
                },
                "sku": {"capacity": 0, "name": "S0", "tier": "Standard"},
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "type": "str",
            },
            api_version="2023-12-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_list(self, resource_group):
        response = self.client.deployments.list(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            api_version="2023-12-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_list_for_cluster(self, resource_group):
        response = self.client.deployments.list_for_cluster(
            resource_group_name=resource_group.name,
            service_name="str",
            api_version="2023-12-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_start(self, resource_group):
        response = self.client.deployments.begin_start(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            deployment_name="str",
            api_version="2023-12-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_stop(self, resource_group):
        response = self.client.deployments.begin_stop(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            deployment_name="str",
            api_version="2023-12-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_restart(self, resource_group):
        response = self.client.deployments.begin_restart(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            deployment_name="str",
            api_version="2023-12-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_enable_remote_debugging(self, resource_group):
        response = self.client.deployments.begin_enable_remote_debugging(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            deployment_name="str",
            api_version="2023-12-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_disable_remote_debugging(self, resource_group):
        response = self.client.deployments.begin_disable_remote_debugging(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            deployment_name="str",
            api_version="2023-12-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_get_remote_debugging_config(self, resource_group):
        response = self.client.deployments.get_remote_debugging_config(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            deployment_name="str",
            api_version="2023-12-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_get_log_file_url(self, resource_group):
        response = self.client.deployments.get_log_file_url(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            deployment_name="str",
            api_version="2023-12-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_generate_heap_dump(self, resource_group):
        response = self.client.deployments.begin_generate_heap_dump(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            deployment_name="str",
            diagnostic_parameters={"appInstance": "str", "duration": "str", "filePath": "str"},
            api_version="2023-12-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_generate_thread_dump(self, resource_group):
        response = self.client.deployments.begin_generate_thread_dump(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            deployment_name="str",
            diagnostic_parameters={"appInstance": "str", "duration": "str", "filePath": "str"},
            api_version="2023-12-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_start_jfr(self, resource_group):
        response = self.client.deployments.begin_start_jfr(
            resource_group_name=resource_group.name,
            service_name="str",
            app_name="str",
            deployment_name="str",
            diagnostic_parameters={"appInstance": "str", "duration": "str", "filePath": "str"},
            api_version="2023-12-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# pylint: disable=too-many-lines
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import sys
from typing import Any, List, Optional, TYPE_CHECKING

from ... import _serialization

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore  # pylint: disable=ungrouped-imports

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from .. import models as _models
JSON = MutableMapping[str, Any]  # pylint: disable=unsubscriptable-object


class ClassicAdministrator(_serialization.Model):
    """Classic Administrators.

    :ivar id: The ID of the administrator.
    :vartype id: str
    :ivar name: The name of the administrator.
    :vartype name: str
    :ivar type: The type of the administrator.
    :vartype type: str
    :ivar email_address: The email address of the administrator.
    :vartype email_address: str
    :ivar role: The role of the administrator.
    :vartype role: str
    """

    _attribute_map = {
        "id": {"key": "id", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "type": {"key": "type", "type": "str"},
        "email_address": {"key": "properties.emailAddress", "type": "str"},
        "role": {"key": "properties.role", "type": "str"},
    }

    def __init__(
        self,
        *,
        id: Optional[str] = None,  # pylint: disable=redefined-builtin
        name: Optional[str] = None,
        type: Optional[str] = None,
        email_address: Optional[str] = None,
        role: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword id: The ID of the administrator.
        :paramtype id: str
        :keyword name: The name of the administrator.
        :paramtype name: str
        :keyword type: The type of the administrator.
        :paramtype type: str
        :keyword email_address: The email address of the administrator.
        :paramtype email_address: str
        :keyword role: The role of the administrator.
        :paramtype role: str
        """
        super().__init__(**kwargs)
        self.id = id
        self.name = name
        self.type = type
        self.email_address = email_address
        self.role = role


class ClassicAdministratorListResult(_serialization.Model):
    """ClassicAdministrator list result information.

    :ivar value: An array of administrators.
    :vartype value: list[~azure.mgmt.authorization.v2015_07_01.models.ClassicAdministrator]
    :ivar next_link: The URL to use for getting the next set of results.
    :vartype next_link: str
    """

    _attribute_map = {
        "value": {"key": "value", "type": "[ClassicAdministrator]"},
        "next_link": {"key": "nextLink", "type": "str"},
    }

    def __init__(
        self,
        *,
        value: Optional[List["_models.ClassicAdministrator"]] = None,
        next_link: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword value: An array of administrators.
        :paramtype value: list[~azure.mgmt.authorization.v2015_07_01.models.ClassicAdministrator]
        :keyword next_link: The URL to use for getting the next set of results.
        :paramtype next_link: str
        """
        super().__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class ErrorAdditionalInfo(_serialization.Model):
    """The resource management error additional info.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar type: The additional info type.
    :vartype type: str
    :ivar info: The additional info.
    :vartype info: JSON
    """

    _validation = {
        "type": {"readonly": True},
        "info": {"readonly": True},
    }

    _attribute_map = {
        "type": {"key": "type", "type": "str"},
        "info": {"key": "info", "type": "object"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.type = None
        self.info = None


class ErrorDetail(_serialization.Model):
    """The error detail.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar code: The error code.
    :vartype code: str
    :ivar message: The error message.
    :vartype message: str
    :ivar target: The error target.
    :vartype target: str
    :ivar details: The error details.
    :vartype details: list[~azure.mgmt.authorization.v2015_07_01.models.ErrorDetail]
    :ivar additional_info: The error additional info.
    :vartype additional_info:
     list[~azure.mgmt.authorization.v2015_07_01.models.ErrorAdditionalInfo]
    """

    _validation = {
        "code": {"readonly": True},
        "message": {"readonly": True},
        "target": {"readonly": True},
        "details": {"readonly": True},
        "additional_info": {"readonly": True},
    }

    _attribute_map = {
        "code": {"key": "code", "type": "str"},
        "message": {"key": "message", "type": "str"},
        "target": {"key": "target", "type": "str"},
        "details": {"key": "details", "type": "[ErrorDetail]"},
        "additional_info": {"key": "additionalInfo", "type": "[ErrorAdditionalInfo]"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.code = None
        self.message = None
        self.target = None
        self.details = None
        self.additional_info = None


class ErrorResponse(_serialization.Model):
    """Common error response for all Azure Resource Manager APIs to return error details for failed
    operations. (This also follows the OData error response format.).

    :ivar error: The error object.
    :vartype error: ~azure.mgmt.authorization.v2015_07_01.models.ErrorDetail
    """

    _attribute_map = {
        "error": {"key": "error", "type": "ErrorDetail"},
    }

    def __init__(self, *, error: Optional["_models.ErrorDetail"] = None, **kwargs: Any) -> None:
        """
        :keyword error: The error object.
        :paramtype error: ~azure.mgmt.authorization.v2015_07_01.models.ErrorDetail
        """
        super().__init__(**kwargs)
        self.error = error


class Permission(_serialization.Model):
    """Role definition permissions.

    :ivar actions: Allowed actions.
    :vartype actions: list[str]
    :ivar not_actions: Denied actions.
    :vartype not_actions: list[str]
    """

    _attribute_map = {
        "actions": {"key": "actions", "type": "[str]"},
        "not_actions": {"key": "notActions", "type": "[str]"},
    }

    def __init__(
        self, *, actions: Optional[List[str]] = None, not_actions: Optional[List[str]] = None, **kwargs: Any
    ) -> None:
        """
        :keyword actions: Allowed actions.
        :paramtype actions: list[str]
        :keyword not_actions: Denied actions.
        :paramtype not_actions: list[str]
        """
        super().__init__(**kwargs)
        self.actions = actions
        self.not_actions = not_actions


class PermissionGetResult(_serialization.Model):
    """Permissions information.

    :ivar value: An array of permissions.
    :vartype value: list[~azure.mgmt.authorization.v2015_07_01.models.Permission]
    :ivar next_link: The URL to use for getting the next set of results.
    :vartype next_link: str
    """

    _attribute_map = {
        "value": {"key": "value", "type": "[Permission]"},
        "next_link": {"key": "nextLink", "type": "str"},
    }

    def __init__(
        self, *, value: Optional[List["_models.Permission"]] = None, next_link: Optional[str] = None, **kwargs: Any
    ) -> None:
        """
        :keyword value: An array of permissions.
        :paramtype value: list[~azure.mgmt.authorization.v2015_07_01.models.Permission]
        :keyword next_link: The URL to use for getting the next set of results.
        :paramtype next_link: str
        """
        super().__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class ProviderOperation(_serialization.Model):
    """Operation.

    :ivar name: The operation name.
    :vartype name: str
    :ivar display_name: The operation display name.
    :vartype display_name: str
    :ivar description: The operation description.
    :vartype description: str
    :ivar origin: The operation origin.
    :vartype origin: str
    :ivar properties: The operation properties.
    :vartype properties: JSON
    """

    _attribute_map = {
        "name": {"key": "name", "type": "str"},
        "display_name": {"key": "displayName", "type": "str"},
        "description": {"key": "description", "type": "str"},
        "origin": {"key": "origin", "type": "str"},
        "properties": {"key": "properties", "type": "object"},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        display_name: Optional[str] = None,
        description: Optional[str] = None,
        origin: Optional[str] = None,
        properties: Optional[JSON] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword name: The operation name.
        :paramtype name: str
        :keyword display_name: The operation display name.
        :paramtype display_name: str
        :keyword description: The operation description.
        :paramtype description: str
        :keyword origin: The operation origin.
        :paramtype origin: str
        :keyword properties: The operation properties.
        :paramtype properties: JSON
        """
        super().__init__(**kwargs)
        self.name = name
        self.display_name = display_name
        self.description = description
        self.origin = origin
        self.properties = properties


class ProviderOperationsMetadata(_serialization.Model):
    """Provider Operations metadata.

    :ivar id: The provider id.
    :vartype id: str
    :ivar name: The provider name.
    :vartype name: str
    :ivar type: The provider type.
    :vartype type: str
    :ivar display_name: The provider display name.
    :vartype display_name: str
    :ivar resource_types: The provider resource types.
    :vartype resource_types: list[~azure.mgmt.authorization.v2015_07_01.models.ResourceType]
    :ivar operations: The provider operations.
    :vartype operations: list[~azure.mgmt.authorization.v2015_07_01.models.ProviderOperation]
    """

    _attribute_map = {
        "id": {"key": "id", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "type": {"key": "type", "type": "str"},
        "display_name": {"key": "displayName", "type": "str"},
        "resource_types": {"key": "resourceTypes", "type": "[ResourceType]"},
        "operations": {"key": "operations", "type": "[ProviderOperation]"},
    }

    def __init__(
        self,
        *,
        id: Optional[str] = None,  # pylint: disable=redefined-builtin
        name: Optional[str] = None,
        type: Optional[str] = None,
        display_name: Optional[str] = None,
        resource_types: Optional[List["_models.ResourceType"]] = None,
        operations: Optional[List["_models.ProviderOperation"]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword id: The provider id.
        :paramtype id: str
        :keyword name: The provider name.
        :paramtype name: str
        :keyword type: The provider type.
        :paramtype type: str
        :keyword display_name: The provider display name.
        :paramtype display_name: str
        :keyword resource_types: The provider resource types.
        :paramtype resource_types: list[~azure.mgmt.authorization.v2015_07_01.models.ResourceType]
        :keyword operations: The provider operations.
        :paramtype operations: list[~azure.mgmt.authorization.v2015_07_01.models.ProviderOperation]
        """
        super().__init__(**kwargs)
        self.id = id
        self.name = name
        self.type = type
        self.display_name = display_name
        self.resource_types = resource_types
        self.operations = operations


class ProviderOperationsMetadataListResult(_serialization.Model):
    """Provider operations metadata list.

    :ivar value: The list of providers.
    :vartype value: list[~azure.mgmt.authorization.v2015_07_01.models.ProviderOperationsMetadata]
    :ivar next_link: The URL to use for getting the next set of results.
    :vartype next_link: str
    """

    _attribute_map = {
        "value": {"key": "value", "type": "[ProviderOperationsMetadata]"},
        "next_link": {"key": "nextLink", "type": "str"},
    }

    def __init__(
        self,
        *,
        value: Optional[List["_models.ProviderOperationsMetadata"]] = None,
        next_link: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword value: The list of providers.
        :paramtype value: list[~azure.mgmt.authorization.v2015_07_01.models.ProviderOperationsMetadata]
        :keyword next_link: The URL to use for getting the next set of results.
        :paramtype next_link: str
        """
        super().__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class ResourceType(_serialization.Model):
    """Resource Type.

    :ivar name: The resource type name.
    :vartype name: str
    :ivar display_name: The resource type display name.
    :vartype display_name: str
    :ivar operations: The resource type operations.
    :vartype operations: list[~azure.mgmt.authorization.v2015_07_01.models.ProviderOperation]
    """

    _attribute_map = {
        "name": {"key": "name", "type": "str"},
        "display_name": {"key": "displayName", "type": "str"},
        "operations": {"key": "operations", "type": "[ProviderOperation]"},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        display_name: Optional[str] = None,
        operations: Optional[List["_models.ProviderOperation"]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword name: The resource type name.
        :paramtype name: str
        :keyword display_name: The resource type display name.
        :paramtype display_name: str
        :keyword operations: The resource type operations.
        :paramtype operations: list[~azure.mgmt.authorization.v2015_07_01.models.ProviderOperation]
        """
        super().__init__(**kwargs)
        self.name = name
        self.display_name = display_name
        self.operations = operations


class RoleAssignment(_serialization.Model):
    """Role Assignments.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: The role assignment ID.
    :vartype id: str
    :ivar name: The role assignment name.
    :vartype name: str
    :ivar type: The role assignment type.
    :vartype type: str
    :ivar properties: Role assignment properties.
    :vartype properties:
     ~azure.mgmt.authorization.v2015_07_01.models.RoleAssignmentPropertiesWithScope
    """

    _validation = {
        "id": {"readonly": True},
        "name": {"readonly": True},
        "type": {"readonly": True},
    }

    _attribute_map = {
        "id": {"key": "id", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "type": {"key": "type", "type": "str"},
        "properties": {"key": "properties", "type": "RoleAssignmentPropertiesWithScope"},
    }

    def __init__(
        self, *, properties: Optional["_models.RoleAssignmentPropertiesWithScope"] = None, **kwargs: Any
    ) -> None:
        """
        :keyword properties: Role assignment properties.
        :paramtype properties:
         ~azure.mgmt.authorization.v2015_07_01.models.RoleAssignmentPropertiesWithScope
        """
        super().__init__(**kwargs)
        self.id = None
        self.name = None
        self.type = None
        self.properties = properties


class RoleAssignmentCreateParameters(_serialization.Model):
    """Role assignment create parameters.

    All required parameters must be populated in order to send to Azure.

    :ivar properties: Role assignment properties. Required.
    :vartype properties: ~azure.mgmt.authorization.v2015_07_01.models.RoleAssignmentProperties
    """

    _validation = {
        "properties": {"required": True},
    }

    _attribute_map = {
        "properties": {"key": "properties", "type": "RoleAssignmentProperties"},
    }

    def __init__(self, *, properties: "_models.RoleAssignmentProperties", **kwargs: Any) -> None:
        """
        :keyword properties: Role assignment properties. Required.
        :paramtype properties: ~azure.mgmt.authorization.v2015_07_01.models.RoleAssignmentProperties
        """
        super().__init__(**kwargs)
        self.properties = properties


class RoleAssignmentFilter(_serialization.Model):
    """Role Assignments filter.

    :ivar principal_id: Returns role assignment of the specific principal.
    :vartype principal_id: str
    """

    _attribute_map = {
        "principal_id": {"key": "principalId", "type": "str"},
    }

    def __init__(self, *, principal_id: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword principal_id: Returns role assignment of the specific principal.
        :paramtype principal_id: str
        """
        super().__init__(**kwargs)
        self.principal_id = principal_id


class RoleAssignmentListResult(_serialization.Model):
    """Role assignment list operation result.

    :ivar value: Role assignment list.
    :vartype value: list[~azure.mgmt.authorization.v2015_07_01.models.RoleAssignment]
    :ivar next_link: The URL to use for getting the next set of results.
    :vartype next_link: str
    """

    _attribute_map = {
        "value": {"key": "value", "type": "[RoleAssignment]"},
        "next_link": {"key": "nextLink", "type": "str"},
    }

    def __init__(
        self, *, value: Optional[List["_models.RoleAssignment"]] = None, next_link: Optional[str] = None, **kwargs: Any
    ) -> None:
        """
        :keyword value: Role assignment list.
        :paramtype value: list[~azure.mgmt.authorization.v2015_07_01.models.RoleAssignment]
        :keyword next_link: The URL to use for getting the next set of results.
        :paramtype next_link: str
        """
        super().__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class RoleAssignmentProperties(_serialization.Model):
    """Role assignment properties.

    All required parameters must be populated in order to send to Azure.

    :ivar role_definition_id: The role definition ID used in the role assignment. Required.
    :vartype role_definition_id: str
    :ivar principal_id: The principal ID assigned to the role. This maps to the ID inside the
     Active Directory. It can point to a user, service principal, or security group. Required.
    :vartype principal_id: str
    """

    _validation = {
        "role_definition_id": {"required": True},
        "principal_id": {"required": True},
    }

    _attribute_map = {
        "role_definition_id": {"key": "roleDefinitionId", "type": "str"},
        "principal_id": {"key": "principalId", "type": "str"},
    }

    def __init__(self, *, role_definition_id: str, principal_id: str, **kwargs: Any) -> None:
        """
        :keyword role_definition_id: The role definition ID used in the role assignment. Required.
        :paramtype role_definition_id: str
        :keyword principal_id: The principal ID assigned to the role. This maps to the ID inside the
         Active Directory. It can point to a user, service principal, or security group. Required.
        :paramtype principal_id: str
        """
        super().__init__(**kwargs)
        self.role_definition_id = role_definition_id
        self.principal_id = principal_id


class RoleAssignmentPropertiesWithScope(_serialization.Model):
    """Role assignment properties with scope.

    :ivar scope: The role assignment scope.
    :vartype scope: str
    :ivar role_definition_id: The role definition ID.
    :vartype role_definition_id: str
    :ivar principal_id: The principal ID.
    :vartype principal_id: str
    """

    _attribute_map = {
        "scope": {"key": "scope", "type": "str"},
        "role_definition_id": {"key": "roleDefinitionId", "type": "str"},
        "principal_id": {"key": "principalId", "type": "str"},
    }

    def __init__(
        self,
        *,
        scope: Optional[str] = None,
        role_definition_id: Optional[str] = None,
        principal_id: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword scope: The role assignment scope.
        :paramtype scope: str
        :keyword role_definition_id: The role definition ID.
        :paramtype role_definition_id: str
        :keyword principal_id: The principal ID.
        :paramtype principal_id: str
        """
        super().__init__(**kwargs)
        self.scope = scope
        self.role_definition_id = role_definition_id
        self.principal_id = principal_id


class RoleDefinition(_serialization.Model):
    """Role definition.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: The role definition ID.
    :vartype id: str
    :ivar name: The role definition name.
    :vartype name: str
    :ivar type: The role definition type.
    :vartype type: str
    :ivar role_name: The role name.
    :vartype role_name: str
    :ivar description: The role definition description.
    :vartype description: str
    :ivar role_type: The role type.
    :vartype role_type: str
    :ivar permissions: Role definition permissions.
    :vartype permissions: list[~azure.mgmt.authorization.v2015_07_01.models.Permission]
    :ivar assignable_scopes: Role definition assignable scopes.
    :vartype assignable_scopes: list[str]
    """

    _validation = {
        "id": {"readonly": True},
        "name": {"readonly": True},
        "type": {"readonly": True},
    }

    _attribute_map = {
        "id": {"key": "id", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "type": {"key": "type", "type": "str"},
        "role_name": {"key": "properties.roleName", "type": "str"},
        "description": {"key": "properties.description", "type": "str"},
        "role_type": {"key": "properties.type", "type": "str"},
        "permissions": {"key": "properties.permissions", "type": "[Permission]"},
        "assignable_scopes": {"key": "properties.assignableScopes", "type": "[str]"},
    }

    def __init__(
        self,
        *,
        role_name: Optional[str] = None,
        description: Optional[str] = None,
        role_type: Optional[str] = None,
        permissions: Optional[List["_models.Permission"]] = None,
        assignable_scopes: Optional[List[str]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword role_name: The role name.
        :paramtype role_name: str
        :keyword description: The role definition description.
        :paramtype description: str
        :keyword role_type: The role type.
        :paramtype role_type: str
        :keyword permissions: Role definition permissions.
        :paramtype permissions: list[~azure.mgmt.authorization.v2015_07_01.models.Permission]
        :keyword assignable_scopes: Role definition assignable scopes.
        :paramtype assignable_scopes: list[str]
        """
        super().__init__(**kwargs)
        self.id = None
        self.name = None
        self.type = None
        self.role_name = role_name
        self.description = description
        self.role_type = role_type
        self.permissions = permissions
        self.assignable_scopes = assignable_scopes


class RoleDefinitionFilter(_serialization.Model):
    """Role Definitions filter.

    :ivar role_name: Returns role definition with the specific name.
    :vartype role_name: str
    """

    _attribute_map = {
        "role_name": {"key": "roleName", "type": "str"},
    }

    def __init__(self, *, role_name: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword role_name: Returns role definition with the specific name.
        :paramtype role_name: str
        """
        super().__init__(**kwargs)
        self.role_name = role_name


class RoleDefinitionListResult(_serialization.Model):
    """Role definition list operation result.

    :ivar value: Role definition list.
    :vartype value: list[~azure.mgmt.authorization.v2015_07_01.models.RoleDefinition]
    :ivar next_link: The URL to use for getting the next set of results.
    :vartype next_link: str
    """

    _attribute_map = {
        "value": {"key": "value", "type": "[RoleDefinition]"},
        "next_link": {"key": "nextLink", "type": "str"},
    }

    def __init__(
        self, *, value: Optional[List["_models.RoleDefinition"]] = None, next_link: Optional[str] = None, **kwargs: Any
    ) -> None:
        """
        :keyword value: Role definition list.
        :paramtype value: list[~azure.mgmt.authorization.v2015_07_01.models.RoleDefinition]
        :keyword next_link: The URL to use for getting the next set of results.
        :paramtype next_link: str
        """
        super().__init__(**kwargs)
        self.value = value
        self.next_link = next_link



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    CommunicationError,
    CommunicationErrorResponse,
    OperatorDetails,
    OperatorInformation,
    OperatorInformationOptions,
    OperatorInformationRequest,
    OperatorInformationResult,
    PhoneNumberAdministrativeDivision,
    PhoneNumberAreaCode,
    PhoneNumberCapabilities,
    PhoneNumberCapabilitiesRequest,
    PhoneNumberCost,
    PhoneNumberCountry,
    PhoneNumberLocality,
    PhoneNumberOffering,
    PhoneNumberOperation,
    PhoneNumberPurchaseRequest,
    PhoneNumberSearchRequest,
    PhoneNumberSearchResult,
    PurchasedPhoneNumber,
)

from ._enums import (  # type: ignore
    BillingFrequency,
    OperatorNumberType,
    PhoneNumberAssignmentType,
    PhoneNumberCapabilityType,
    PhoneNumberOperationStatus,
    PhoneNumberOperationType,
    PhoneNumberSearchResultError,
    PhoneNumberType,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "CommunicationError",
    "CommunicationErrorResponse",
    "OperatorDetails",
    "OperatorInformation",
    "OperatorInformationOptions",
    "OperatorInformationRequest",
    "OperatorInformationResult",
    "PhoneNumberAdministrativeDivision",
    "PhoneNumberAreaCode",
    "PhoneNumberCapabilities",
    "PhoneNumberCapabilitiesRequest",
    "PhoneNumberCost",
    "PhoneNumberCountry",
    "PhoneNumberLocality",
    "PhoneNumberOffering",
    "PhoneNumberOperation",
    "PhoneNumberPurchaseRequest",
    "PhoneNumberSearchRequest",
    "PhoneNumberSearchResult",
    "PurchasedPhoneNumber",
    "BillingFrequency",
    "OperatorNumberType",
    "PhoneNumberAssignmentType",
    "PhoneNumberCapabilityType",
    "PhoneNumberOperationStatus",
    "PhoneNumberOperationType",
    "PhoneNumberSearchResultError",
    "PhoneNumberType",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
import os

COMMUNICATION_CUSTOM_ENDPOINT_ENABLED = "COMMUNICATION_CUSTOM_ENDPOINT_ENABLED"
COMMUNICATION_CUSTOM_URL = "COMMUNICATION_CUSTOM_URL"


def get_custom_enabled():
    """From environment variable, get whether to use a custom endpoint.
    This is only for development purposes.

    :return: custom_enabled
    :rtype: bool
    """
    custom_enabled_str = get_environment_variable(COMMUNICATION_CUSTOM_ENDPOINT_ENABLED)

    if custom_enabled_str is not None:
        return custom_enabled_str.lower() == "true"
    return False


def get_custom_url():
    """From environment variable, get the acs endpoint to call.
    This is only for development purposes.

    :return: custom_url
    :rtype: str
    """
    return get_environment_variable(COMMUNICATION_CUSTOM_URL)


def get_environment_variable(variable_name):
    """From environment variable, from variable name.
    This is only for development purposes

    :param variable_name: key of the environment variable
    :type variable_name: str
    :return: environment variable value
    :rtype: str
    """
    return os.environ.get(variable_name)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential
from azure.mgmt.costmanagement import CostManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-costmanagement
# USAGE
    python external_billing_account_forecast.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = CostManagementClient(
        credential=DefaultAzureCredential(),
    )

    response = client.forecast.external_cloud_provider_usage(
        external_cloud_provider_type="externalBillingAccounts",
        external_cloud_provider_id="100",
        parameters={
            "dataset": {
                "aggregation": {"totalCost": {"function": "Sum", "name": "Cost"}},
                "filter": {
                    "and": [
                        {
                            "or": [
                                {
                                    "dimensions": {
                                        "name": "ResourceLocation",
                                        "operator": "In",
                                        "values": ["East US", "West Europe"],
                                    }
                                },
                                {"tags": {"name": "Environment", "operator": "In", "values": ["UAT", "Prod"]}},
                            ]
                        },
                        {"dimensions": {"name": "ResourceGroup", "operator": "In", "values": ["API"]}},
                    ]
                },
                "granularity": "Daily",
            },
            "timePeriod": {"from": "2022-08-01T00:00:00+00:00", "to": "2022-08-31T23:59:59+00:00"},
            "timeframe": "Custom",
            "type": "Usage",
        },
    )
    print(response)


# x-ms-original-file: specification/cost-management/resource-manager/Microsoft.CostManagement/stable/2022-10-01/examples/ExternalBillingAccountForecast.json
if __name__ == "__main__":
    main()



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import sys
from typing import Any, Callable, Dict, Optional, TypeVar

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._cloud_manifest_file_operations import build_get_request, build_list_request

if sys.version_info >= (3, 8):
    from typing import Literal  # pylint: disable=no-name-in-module, ungrouped-imports
else:
    from typing_extensions import Literal  # type: ignore  # pylint: disable=ungrouped-imports
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class CloudManifestFileOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.azurestack.aio.AzureStackManagementClient`'s
        :attr:`cloud_manifest_file` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace_async
    async def list(self, **kwargs: Any) -> _models.CloudManifestFileResponse:
        """Returns a cloud specific manifest JSON file with latest version.

        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CloudManifestFileResponse or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.CloudManifestFileResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2022-06-01"] = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )
        cls: ClsType[_models.CloudManifestFileResponse] = kwargs.pop("cls", None)

        request = build_list_request(
            api_version=api_version,
            template_url=self.list.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("CloudManifestFileResponse", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list.metadata = {"url": "/providers/Microsoft.AzureStack/cloudManifestFiles"}

    @distributed_trace_async
    async def get(
        self, verification_version: str, version_creation_date: Optional[str] = None, **kwargs: Any
    ) -> _models.CloudManifestFileResponse:
        """Returns a cloud specific manifest JSON file.

        :param verification_version: Signing verification key version. Required.
        :type verification_version: str
        :param version_creation_date: Signing verification key version creation date. Default value is
         None.
        :type version_creation_date: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CloudManifestFileResponse or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.CloudManifestFileResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2022-06-01"] = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )
        cls: ClsType[_models.CloudManifestFileResponse] = kwargs.pop("cls", None)

        request = build_get_request(
            verification_version=verification_version,
            version_creation_date=version_creation_date,
            api_version=api_version,
            template_url=self.get.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("CloudManifestFileResponse", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get.metadata = {"url": "/providers/Microsoft.AzureStack/cloudManifestFiles/{verificationVersion}"}



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
import os
from devtools_testutils import is_live


def email_decorator(func, **kwargs):
    def wrapper(self, *args, **kwargs):
        if is_live():
            self.communication_connection_string = os.environ["COMMUNICATION_CONNECTION_STRING_EMAIL"]
            self.sender_address = os.environ["SENDER_ADDRESS"]
            self.recipient_address = os.environ["RECIPIENT_ADDRESS"]
        else:
            self.communication_connection_string = "endpoint=https://someEndpoint/;accesskey=someAccessKeyw=="
            self.sender_address = "someSender@contoso.com"
            self.recipient_address = "someRecipient@domain.com"

        func(self, *args, **kwargs)

    return wrapper



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import sys
from typing import Any, Callable, Dict, Optional, TypeVar

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models
from ...operations._live_token_operations import build_get_request

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class LiveTokenOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.applicationinsights.v2021_10.aio.ApplicationInsightsManagementClient`'s
        :attr:`live_token` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")
        self._api_version = input_args.pop(0) if input_args else kwargs.pop("api_version")

    @distributed_trace_async
    async def get(self, resource_uri: str, **kwargs: Any) -> _models.LiveTokenResponse:
        """**Gets an access token for live metrics stream data.**.

        :param resource_uri: The identifier of the resource. Required.
        :type resource_uri: str
        :return: LiveTokenResponse or the result of cls(response)
        :rtype: ~azure.mgmt.applicationinsights.v2021_10.models.LiveTokenResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._api_version or "2021-10-14"))
        cls: ClsType[_models.LiveTokenResponse] = kwargs.pop("cls", None)

        _request = build_get_request(
            resource_uri=resource_uri,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseLinkedStorage, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("LiveTokenResponse", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore



# coding=utf-8
# pylint: disable=too-many-lines
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import datetime
import sys
from typing import Any, List, Mapping, Optional, TYPE_CHECKING, Union, overload

from .. import _model_base
from .._model_base import rest_field

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore  # pylint: disable=ungrouped-imports

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from .. import models as _models
JSON = MutableMapping[str, Any]  # pylint: disable=unsubscriptable-object


class AlignPolicy(_model_base.Model):
    """An optional field, indicating the manner to align multiple variables.

    :ivar align_mode: An optional field, indicating how to align different variables to the same
     time-range. Either Inner or Outer. Known values are: "Inner" and "Outer".
    :vartype align_mode: str or ~anomalydetector.models.AlignMode
    :ivar fill_n_a_method: An optional field, indicating how missing values will be filled. One of
     Previous, Subsequent, Linear, Zero, Fixed. Known values are: "Previous", "Subsequent",
     "Linear", "Zero", and "Fixed".
    :vartype fill_n_a_method: str or ~anomalydetector.models.FillNAMethod
    :ivar padding_value: An optional field. Required when fillNAMethod is Fixed.
    :vartype padding_value: float
    """

    align_mode: Optional[Union[str, "_models.AlignMode"]] = rest_field(name="alignMode")
    """An optional field, indicating how to align different variables to the same
time-range. Either Inner or Outer. Known values are: \"Inner\" and \"Outer\"."""
    fill_n_a_method: Optional[Union[str, "_models.FillNAMethod"]] = rest_field(name="fillNAMethod")
    """An optional field, indicating how missing values will be filled. One of
Previous, Subsequent, Linear, Zero, Fixed. Known values are: \"Previous\", \"Subsequent\", \"Linear\", \"Zero\", and \"Fixed\"."""
    padding_value: Optional[float] = rest_field(name="paddingValue")
    """An optional field. Required when fillNAMethod is Fixed. """

    @overload
    def __init__(
        self,
        *,
        align_mode: Optional[Union[str, "_models.AlignMode"]] = None,
        fill_n_a_method: Optional[Union[str, "_models.FillNAMethod"]] = None,
        padding_value: Optional[float] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class AnomalyDetectionModel(_model_base.Model):
    """Response of getting a model.

    Readonly variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar model_id: Model identifier. Required.
    :vartype model_id: str
    :ivar created_time: Date and time (UTC) when the model was created. Required.
    :vartype created_time: ~datetime.datetime
    :ivar last_updated_time: Date and time (UTC) when the model was last updated. Required.
    :vartype last_updated_time: ~datetime.datetime
    :ivar model_info: Training result of a model including its status, errors and diagnostics
     information.
    :vartype model_info: ~anomalydetector.models.ModelInfo
    """

    model_id: str = rest_field(name="modelId", readonly=True)
    """Model identifier. Required. """
    created_time: datetime.datetime = rest_field(name="createdTime")
    """Date and time (UTC) when the model was created. Required. """
    last_updated_time: datetime.datetime = rest_field(name="lastUpdatedTime")
    """Date and time (UTC) when the model was last updated. Required. """
    model_info: Optional["_models.ModelInfo"] = rest_field(name="modelInfo")
    """Training result of a model including its status, errors and diagnostics
information. """

    @overload
    def __init__(
        self,
        *,
        created_time: datetime.datetime,
        last_updated_time: datetime.datetime,
        model_info: Optional["_models.ModelInfo"] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class AnomalyDetectorError(_model_base.Model):
    """Error information returned by the API.

    :ivar x_ms_error_code: Error code.
    :vartype x_ms_error_code: str
    :ivar code: The error code. Known values are: "InvalidCustomInterval", "BadArgument",
     "InvalidGranularity", "InvalidPeriod", "InvalidModelArgument", "InvalidSeries",
     "InvalidJsonFormat", "RequiredGranularity", "RequiredSeries", "InvalidImputeMode", and
     "InvalidImputeFixedValue".
    :vartype code: str or ~anomalydetector.models.AnomalyDetectorErrorCodes
    :ivar message: A message explaining the error reported by the service.
    :vartype message: str
    """

    x_ms_error_code: Optional[str] = rest_field(name="x-ms-error-code")
    """Error code. """
    code: Optional[Union[str, "_models.AnomalyDetectorErrorCodes"]] = rest_field()
    """The error code. Known values are: \"InvalidCustomInterval\", \"BadArgument\", \"InvalidGranularity\", \"InvalidPeriod\", \"InvalidModelArgument\", \"InvalidSeries\", \"InvalidJsonFormat\", \"RequiredGranularity\", \"RequiredSeries\", \"InvalidImputeMode\", and \"InvalidImputeFixedValue\"."""
    message: Optional[str] = rest_field()
    """A message explaining the error reported by the service. """

    @overload
    def __init__(
        self,
        *,
        x_ms_error_code: Optional[str] = None,
        code: Optional[Union[str, "_models.AnomalyDetectorErrorCodes"]] = None,
        message: Optional[str] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class AnomalyInterpretation(_model_base.Model):
    """Interpretation of the anomalous timestamp.

    :ivar variable: Variable.
    :vartype variable: str
    :ivar contribution_score: This score shows the percentage contributing to the anomalous
     timestamp. A
     number between 0 and 1.
    :vartype contribution_score: float
    :ivar correlation_changes: Correlation changes among the anomalous variables.
    :vartype correlation_changes: ~anomalydetector.models.CorrelationChanges
    """

    variable: Optional[str] = rest_field()
    """Variable. """
    contribution_score: Optional[float] = rest_field(name="contributionScore")
    """This score shows the percentage contributing to the anomalous timestamp. A
number between 0 and 1. """
    correlation_changes: Optional["_models.CorrelationChanges"] = rest_field(name="correlationChanges")
    """Correlation changes among the anomalous variables. """

    @overload
    def __init__(
        self,
        *,
        variable: Optional[str] = None,
        contribution_score: Optional[float] = None,
        correlation_changes: Optional["_models.CorrelationChanges"] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class AnomalyState(_model_base.Model):
    """Anomaly status and information.

    All required parameters must be populated in order to send to Azure.

    :ivar timestamp: The timestamp for this anomaly. Required.
    :vartype timestamp: ~datetime.datetime
    :ivar value: The detailed value of this anomalous timestamp.
    :vartype value: ~anomalydetector.models.AnomalyValue
    :ivar errors: Error message for the current timestamp.
    :vartype errors: list[~anomalydetector.models.ErrorResponse]
    """

    timestamp: datetime.datetime = rest_field()
    """The timestamp for this anomaly. Required. """
    value: Optional["_models.AnomalyValue"] = rest_field()
    """The detailed value of this anomalous timestamp. """
    errors: Optional[List["_models.ErrorResponse"]] = rest_field()
    """Error message for the current timestamp. """

    @overload
    def __init__(
        self,
        *,
        timestamp: datetime.datetime,
        value: Optional["_models.AnomalyValue"] = None,
        errors: Optional[List["_models.ErrorResponse"]] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class AnomalyValue(_model_base.Model):
    """Detailed information of the anomalous timestamp.

    All required parameters must be populated in order to send to Azure.

    :ivar is_anomaly: True if an anomaly is detected at the current timestamp. Required.
    :vartype is_anomaly: bool
    :ivar severity: Indicates the significance of the anomaly. The higher the severity, the more
     significant the anomaly is. Required.
    :vartype severity: float
    :ivar score: Raw anomaly score of severity, will help indicate the degree of abnormality as
     well. Required.
    :vartype score: float
    :ivar interpretation: Interpretation of this anomalous timestamp.
    :vartype interpretation: list[~anomalydetector.models.AnomalyInterpretation]
    """

    is_anomaly: bool = rest_field(name="isAnomaly")
    """True if an anomaly is detected at the current timestamp. Required. """
    severity: float = rest_field()
    """Indicates the significance of the anomaly. The higher the severity, the more
significant the anomaly is. Required. """
    score: float = rest_field()
    """Raw anomaly score of severity, will help indicate the degree of abnormality as
well. Required. """
    interpretation: Optional[List["_models.AnomalyInterpretation"]] = rest_field()
    """Interpretation of this anomalous timestamp. """

    @overload
    def __init__(
        self,
        *,
        is_anomaly: bool,
        severity: float,
        score: float,
        interpretation: Optional[List["_models.AnomalyInterpretation"]] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class CorrelationChanges(_model_base.Model):
    """Correlation changes among the anomalous variables.

    :ivar changed_variables: The correlated variables that have correlation changes under an
     anomaly.
    :vartype changed_variables: list[str]
    """

    changed_variables: Optional[List[str]] = rest_field(name="changedVariables")
    """The correlated variables that have correlation changes under an anomaly. """

    @overload
    def __init__(
        self,
        *,
        changed_variables: Optional[List[str]] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class DiagnosticsInfo(_model_base.Model):
    """Diagnostics information to help inspect the states of model or variable.

    :ivar model_state: Model status.
    :vartype model_state: ~anomalydetector.models.ModelState
    :ivar variable_states: Variable Status.
    :vartype variable_states: list[~anomalydetector.models.VariableState]
    """

    model_state: Optional["_models.ModelState"] = rest_field(name="modelState")
    """Model status. """
    variable_states: Optional[List["_models.VariableState"]] = rest_field(name="variableStates")
    """Variable Status. """

    @overload
    def __init__(
        self,
        *,
        model_state: Optional["_models.ModelState"] = None,
        variable_states: Optional[List["_models.VariableState"]] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class ErrorResponse(_model_base.Model):
    """ErrorResponse contains code and message that shows the error information.

    All required parameters must be populated in order to send to Azure.

    :ivar code: The error code. Required.
    :vartype code: str
    :ivar message: The message explaining the error reported by the service. Required.
    :vartype message: str
    """

    code: str = rest_field()
    """The error code. Required. """
    message: str = rest_field()
    """The message explaining the error reported by the service. Required. """

    @overload
    def __init__(
        self,
        *,
        code: str,
        message: str,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class ModelInfo(_model_base.Model):
    """Training result of a model including its status, errors and diagnostics
    information.

    Readonly variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar data_source: Source link to the input data to indicate an accessible Azure storage Uri,
     either pointed to an Azure blob storage folder, or pointed to a CSV file in
     Azure blob storage based on you data schema selection. Required.
    :vartype data_source: str
    :ivar data_schema: Data schema of input data source: OneTable or MultiTable. The default
     DataSchema is OneTable. Known values are: "OneTable" and "MultiTable".
    :vartype data_schema: str or ~anomalydetector.models.DataSchema
    :ivar start_time: A required field, indicating the start time of training data, which should be
     date-time of ISO 8601 format. Required.
    :vartype start_time: ~datetime.datetime
    :ivar end_time: A required field, indicating the end time of training data, which should be
     date-time of ISO 8601 format. Required.
    :vartype end_time: ~datetime.datetime
    :ivar display_name: An optional field. The display name of the model whose maximum length is 24
     characters.
    :vartype display_name: str
    :ivar sliding_window: An optional field, indicating how many previous timestamps will be used
     to
     detect whether the timestamp is anomaly or not.
    :vartype sliding_window: int
    :ivar align_policy: An optional field, indicating the manner to align multiple variables.
    :vartype align_policy: ~anomalydetector.models.AlignPolicy
    :ivar status: Model status. One of CREATED, RUNNING, READY, and FAILED. Known values are:
     "CREATED", "RUNNING", "READY", and "FAILED".
    :vartype status: str or ~anomalydetector.models.ModelStatus
    :ivar errors: Error messages when failed to create a model.
    :vartype errors: list[~anomalydetector.models.ErrorResponse]
    :ivar diagnostics_info: Diagnostics information to help inspect the states of model or
     variable.
    :vartype diagnostics_info: ~anomalydetector.models.DiagnosticsInfo
    """

    data_source: str = rest_field(name="dataSource")
    """Source link to the input data to indicate an accessible Azure storage Uri,
either pointed to an Azure blob storage folder, or pointed to a CSV file in
Azure blob storage based on you data schema selection. Required. """
    data_schema: Optional[Union[str, "_models.DataSchema"]] = rest_field(name="dataSchema")
    """Data schema of input data source: OneTable or MultiTable. The default
DataSchema is OneTable. Known values are: \"OneTable\" and \"MultiTable\"."""
    start_time: datetime.datetime = rest_field(name="startTime")
    """A required field, indicating the start time of training data, which should be
date-time of ISO 8601 format. Required. """
    end_time: datetime.datetime = rest_field(name="endTime")
    """A required field, indicating the end time of training data, which should be
date-time of ISO 8601 format. Required. """
    display_name: Optional[str] = rest_field(name="displayName")
    """An optional field. The display name of the model whose maximum length is 24
characters. """
    sliding_window: Optional[int] = rest_field(name="slidingWindow")
    """An optional field, indicating how many previous timestamps will be used to
detect whether the timestamp is anomaly or not. """
    align_policy: Optional["_models.AlignPolicy"] = rest_field(name="alignPolicy")
    """An optional field, indicating the manner to align multiple variables. """
    status: Optional[Union[str, "_models.ModelStatus"]] = rest_field()
    """Model status. One of CREATED, RUNNING, READY, and FAILED. Known values are: \"CREATED\", \"RUNNING\", \"READY\", and \"FAILED\"."""
    errors: Optional[List["_models.ErrorResponse"]] = rest_field(readonly=True)
    """Error messages when failed to create a model. """
    diagnostics_info: Optional["_models.DiagnosticsInfo"] = rest_field(name="diagnosticsInfo")
    """Diagnostics information to help inspect the states of model or variable. """

    @overload
    def __init__(
        self,
        *,
        data_source: str,
        start_time: datetime.datetime,
        end_time: datetime.datetime,
        data_schema: Optional[Union[str, "_models.DataSchema"]] = None,
        display_name: Optional[str] = None,
        sliding_window: Optional[int] = None,
        align_policy: Optional["_models.AlignPolicy"] = None,
        status: Optional[Union[str, "_models.ModelStatus"]] = None,
        diagnostics_info: Optional["_models.DiagnosticsInfo"] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class ModelList(_model_base.Model):
    """Response of listing models.

    All required parameters must be populated in order to send to Azure.

    :ivar models: List of models. Required.
    :vartype models: list[~anomalydetector.models.AnomalyDetectionModel]
    :ivar current_count: Number of trained multivariate models. Required.
    :vartype current_count: int
    :ivar max_count: Maximum number of models that can be trained for this Anomaly Detector
     resource. Required.
    :vartype max_count: int
    :ivar next_link: The link to fetch more models.
    :vartype next_link: str
    """

    models: List["_models.AnomalyDetectionModel"] = rest_field()
    """List of models. Required. """
    current_count: int = rest_field(name="currentCount")
    """Number of trained multivariate models. Required. """
    max_count: int = rest_field(name="maxCount")
    """Maximum number of models that can be trained for this Anomaly Detector resource. Required. """
    next_link: Optional[str] = rest_field(name="nextLink")
    """The link to fetch more models. """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class ModelState(_model_base.Model):
    """Model status.

    :ivar epoch_ids: This indicates the number of passes of the entire training dataset the
     algorithm has completed.
    :vartype epoch_ids: list[int]
    :ivar train_losses: List of metrics used to assess how the model fits the training data for
     each
     epoch.
    :vartype train_losses: list[float]
    :ivar validation_losses: List of metrics used to assess how the model fits the validation set
     for each
     epoch.
    :vartype validation_losses: list[float]
    :ivar latencies_in_seconds: Latency for each epoch.
    :vartype latencies_in_seconds: list[float]
    """

    epoch_ids: Optional[List[int]] = rest_field(name="epochIds")
    """This indicates the number of passes of the entire training dataset the
algorithm has completed. """
    train_losses: Optional[List[float]] = rest_field(name="trainLosses")
    """List of metrics used to assess how the model fits the training data for each
epoch. """
    validation_losses: Optional[List[float]] = rest_field(name="validationLosses")
    """List of metrics used to assess how the model fits the validation set for each
epoch. """
    latencies_in_seconds: Optional[List[float]] = rest_field(name="latenciesInSeconds")
    """Latency for each epoch. """

    @overload
    def __init__(
        self,
        *,
        epoch_ids: Optional[List[int]] = None,
        train_losses: Optional[List[float]] = None,
        validation_losses: Optional[List[float]] = None,
        latencies_in_seconds: Optional[List[float]] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class MultivariateBatchDetectionOptions(_model_base.Model):
    """Detection request for batch inference. This is an asynchronous inference which
    will need another API to get detection results.

    All required parameters must be populated in order to send to Azure.

    :ivar data_source: Source link to the input data to indicate an accessible Azure storage Uri,
     either pointed to an Azure blob storage folder, or pointed to a CSV file in
     Azure blob storage based on you data schema selection. The data schema should
     be exactly the same with those used in the training phase. Required.
    :vartype data_source: str
    :ivar top_contributor_count: An optional field, which is used to specify the number of top
     contributed
     variables for one anomalous timestamp in the response. The default number is
     10. Required.
    :vartype top_contributor_count: int
    :ivar start_time: A required field, indicating the start time of data for detection, which
     should
     be date-time of ISO 8601 format. Required.
    :vartype start_time: ~datetime.datetime
    :ivar end_time: A required field, indicating the end time of data for detection, which should
     be date-time of ISO 8601 format. Required.
    :vartype end_time: ~datetime.datetime
    """

    data_source: str = rest_field(name="dataSource")
    """Source link to the input data to indicate an accessible Azure storage Uri,
either pointed to an Azure blob storage folder, or pointed to a CSV file in
Azure blob storage based on you data schema selection. The data schema should
be exactly the same with those used in the training phase. Required. """
    top_contributor_count: int = rest_field(name="topContributorCount")
    """An optional field, which is used to specify the number of top contributed
variables for one anomalous timestamp in the response. The default number is
10. Required. """
    start_time: datetime.datetime = rest_field(name="startTime")
    """A required field, indicating the start time of data for detection, which should
be date-time of ISO 8601 format. Required. """
    end_time: datetime.datetime = rest_field(name="endTime")
    """A required field, indicating the end time of data for detection, which should
be date-time of ISO 8601 format. Required. """

    @overload
    def __init__(
        self,
        *,
        data_source: str,
        top_contributor_count: int,
        start_time: datetime.datetime,
        end_time: datetime.datetime,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class MultivariateBatchDetectionResultSummary(_model_base.Model):
    """Multivariate anomaly detection status.

    All required parameters must be populated in order to send to Azure.

    :ivar status: Status of detection results. One of CREATED, RUNNING, READY, and FAILED.
     Required. Known values are: "CREATED", "RUNNING", "READY", and "FAILED".
    :vartype status: str or ~anomalydetector.models.MultivariateBatchDetectionStatus
    :ivar errors: Error message when detection is failed.
    :vartype errors: list[~anomalydetector.models.ErrorResponse]
    :ivar variable_states: Variable Status.
    :vartype variable_states: list[~anomalydetector.models.VariableState]
    :ivar setup_info: Detection request for batch inference. This is an asynchronous inference
     which
     will need another API to get detection results. Required.
    :vartype setup_info: ~anomalydetector.models.MultivariateBatchDetectionOptions
    """

    status: Union[str, "_models.MultivariateBatchDetectionStatus"] = rest_field()
    """Status of detection results. One of CREATED, RUNNING, READY, and FAILED. Required. Known values are: \"CREATED\", \"RUNNING\", \"READY\", and \"FAILED\"."""
    errors: Optional[List["_models.ErrorResponse"]] = rest_field()
    """Error message when detection is failed. """
    variable_states: Optional[List["_models.VariableState"]] = rest_field(name="variableStates")
    """Variable Status. """
    setup_info: "_models.MultivariateBatchDetectionOptions" = rest_field(name="setupInfo")
    """Detection request for batch inference. This is an asynchronous inference which
will need another API to get detection results. Required. """

    @overload
    def __init__(
        self,
        *,
        status: Union[str, "_models.MultivariateBatchDetectionStatus"],
        setup_info: "_models.MultivariateBatchDetectionOptions",
        errors: Optional[List["_models.ErrorResponse"]] = None,
        variable_states: Optional[List["_models.VariableState"]] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class MultivariateDetectionResult(_model_base.Model):
    """Detection results for the given resultId.

    Readonly variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar result_id: Result identifier, which is used to fetch the results of an inference call.
     Required.
    :vartype result_id: str
    :ivar summary: Multivariate anomaly detection status. Required.
    :vartype summary: ~anomalydetector.models.MultivariateBatchDetectionResultSummary
    :ivar results: Detection result for each timestamp. Required.
    :vartype results: list[~anomalydetector.models.AnomalyState]
    """

    result_id: str = rest_field(name="resultId", readonly=True)
    """Result identifier, which is used to fetch the results of an inference call. Required. """
    summary: "_models.MultivariateBatchDetectionResultSummary" = rest_field()
    """Multivariate anomaly detection status. Required. """
    results: List["_models.AnomalyState"] = rest_field()
    """Detection result for each timestamp. Required. """

    @overload
    def __init__(
        self,
        *,
        summary: "_models.MultivariateBatchDetectionResultSummary",
        results: List["_models.AnomalyState"],
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class MultivariateLastDetectionOptions(_model_base.Model):
    """Request of last detection.

    All required parameters must be populated in order to send to Azure.

    :ivar variables: This contains the inference data, including the name, timestamps(ISO 8601) and
     values of variables. Required.
    :vartype variables: list[~anomalydetector.models.VariableValues]
    :ivar top_contributor_count: An optional field, which is used to specify the number of top
     contributed
     variables for one anomalous timestamp in the response. The default number is
     10. Required.
    :vartype top_contributor_count: int
    """

    variables: List["_models.VariableValues"] = rest_field()
    """This contains the inference data, including the name, timestamps(ISO 8601) and
values of variables. Required. """
    top_contributor_count: int = rest_field(name="topContributorCount")
    """An optional field, which is used to specify the number of top contributed
variables for one anomalous timestamp in the response. The default number is
10. Required. """

    @overload
    def __init__(
        self,
        *,
        variables: List["_models.VariableValues"],
        top_contributor_count: int,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class MultivariateLastDetectionResult(_model_base.Model):
    """Results of last detection.

    :ivar variable_states: Variable Status.
    :vartype variable_states: list[~anomalydetector.models.VariableState]
    :ivar results: Anomaly status and information.
    :vartype results: list[~anomalydetector.models.AnomalyState]
    """

    variable_states: Optional[List["_models.VariableState"]] = rest_field(name="variableStates")
    """Variable Status. """
    results: Optional[List["_models.AnomalyState"]] = rest_field()
    """Anomaly status and information. """

    @overload
    def __init__(
        self,
        *,
        variable_states: Optional[List["_models.VariableState"]] = None,
        results: Optional[List["_models.AnomalyState"]] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class ResponseError(_model_base.Model):
    """Error response.

    All required parameters must be populated in order to send to Azure.

    :ivar x_ms_error_code: Error code.
    :vartype x_ms_error_code: str
    :ivar code: The error code. Required.
    :vartype code: str
    :ivar message: The message explaining the error reported by the service. Required.
    :vartype message: str
    """

    x_ms_error_code: Optional[str] = rest_field(name="x-ms-error-code")
    """Error code. """
    code: str = rest_field()
    """The error code. Required. """
    message: str = rest_field()
    """The message explaining the error reported by the service. Required. """

    @overload
    def __init__(
        self,
        *,
        code: str,
        message: str,
        x_ms_error_code: Optional[str] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class TimeSeriesPoint(_model_base.Model):
    """The definition of input timeseries points.

    All required parameters must be populated in order to send to Azure.

    :ivar timestamp: Optional argument, timestamp of a data point (ISO8601 format).
    :vartype timestamp: ~datetime.datetime
    :ivar value: The measurement of that point, should be float. Required.
    :vartype value: float
    """

    timestamp: Optional[datetime.datetime] = rest_field()
    """Optional argument, timestamp of a data point (ISO8601 format). """
    value: float = rest_field()
    """The measurement of that point, should be float. Required. """

    @overload
    def __init__(
        self,
        *,
        value: float,
        timestamp: Optional[datetime.datetime] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class UnivariateChangePointDetectionOptions(_model_base.Model):
    """The request of change point detection.

    All required parameters must be populated in order to send to Azure.

    :ivar series: Time series data points. Points should be sorted by timestamp in ascending
     order to match the change point detection result. Required.
    :vartype series: list[~anomalydetector.models.TimeSeriesPoint]
    :ivar granularity: Can only be one of yearly, monthly, weekly, daily, hourly, minutely or
     secondly. Granularity is used for verify whether input series is valid. Required. Known values
     are: "yearly", "monthly", "weekly", "daily", "hourly", "minutely", "secondly", "microsecond",
     and "none".
    :vartype granularity: str or ~anomalydetector.models.TimeGranularity
    :ivar custom_interval: Custom Interval is used to set non-standard time interval, for example,
     if the
     series is 5 minutes, request can be set as {"granularity":"minutely",
     "customInterval":5}.
    :vartype custom_interval: int
    :ivar period: Optional argument, periodic value of a time series. If the value is null or
     does not present, the API will determine the period automatically.
    :vartype period: int
    :ivar stable_trend_window: Optional argument, advanced model parameter, a default
     stableTrendWindow will
     be used in detection.
    :vartype stable_trend_window: int
    :ivar threshold: Optional argument, advanced model parameter, between 0.0-1.0, the lower the
     value is, the larger the trend error will be which means less change point will
     be accepted.
    :vartype threshold: float
    """

    series: List["_models.TimeSeriesPoint"] = rest_field()
    """Time series data points. Points should be sorted by timestamp in ascending
order to match the change point detection result. Required. """
    granularity: Union[str, "_models.TimeGranularity"] = rest_field()
    """Can only be one of yearly, monthly, weekly, daily, hourly, minutely or
secondly. Granularity is used for verify whether input series is valid. Required. Known values are: \"yearly\", \"monthly\", \"weekly\", \"daily\", \"hourly\", \"minutely\", \"secondly\", \"microsecond\", and \"none\"."""
    custom_interval: Optional[int] = rest_field(name="customInterval")
    """Custom Interval is used to set non-standard time interval, for example, if the
series is 5 minutes, request can be set as {\"granularity\":\"minutely\",
\"customInterval\":5}. """
    period: Optional[int] = rest_field()
    """Optional argument, periodic value of a time series. If the value is null or
does not present, the API will determine the period automatically. """
    stable_trend_window: Optional[int] = rest_field(name="stableTrendWindow")
    """Optional argument, advanced model parameter, a default stableTrendWindow will
be used in detection. """
    threshold: Optional[float] = rest_field()
    """Optional argument, advanced model parameter, between 0.0-1.0, the lower the
value is, the larger the trend error will be which means less change point will
be accepted. """

    @overload
    def __init__(
        self,
        *,
        series: List["_models.TimeSeriesPoint"],
        granularity: Union[str, "_models.TimeGranularity"],
        custom_interval: Optional[int] = None,
        period: Optional[int] = None,
        stable_trend_window: Optional[int] = None,
        threshold: Optional[float] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class UnivariateChangePointDetectionResult(_model_base.Model):
    """The response of change point detection.

    Readonly variables are only populated by the server, and will be ignored when sending a request.

    :ivar period: Frequency extracted from the series, zero means no recurrent pattern has been
     found.
    :vartype period: int
    :ivar is_change_point: isChangePoint contains change point properties for each input point.
     True means
     an anomaly either negative or positive has been detected. The index of the
     array is consistent with the input series.
    :vartype is_change_point: list[bool]
    :ivar confidence_scores: the change point confidence of each point.
    :vartype confidence_scores: list[float]
    """

    period: Optional[int] = rest_field(readonly=True)
    """Frequency extracted from the series, zero means no recurrent pattern has been
found. """
    is_change_point: Optional[List[bool]] = rest_field(name="isChangePoint")
    """isChangePoint contains change point properties for each input point. True means
an anomaly either negative or positive has been detected. The index of the
array is consistent with the input series. """
    confidence_scores: Optional[List[float]] = rest_field(name="confidenceScores")
    """the change point confidence of each point. """

    @overload
    def __init__(
        self,
        *,
        is_change_point: Optional[List[bool]] = None,
        confidence_scores: Optional[List[float]] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class UnivariateDetectionOptions(_model_base.Model):
    """The request of entire or last anomaly detection.

    All required parameters must be populated in order to send to Azure.

    :ivar series: Time series data points. Points should be sorted by timestamp in ascending
     order to match the anomaly detection result. If the data is not sorted
     correctly or there is duplicated timestamp, the API will not work. In such
     case, an error message will be returned. Required.
    :vartype series: list[~anomalydetector.models.TimeSeriesPoint]
    :ivar granularity: Optional argument, can be one of yearly, monthly, weekly, daily, hourly,
     minutely, secondly, microsecond or none. If granularity is not present, it will
     be none by default. If granularity is none, the timestamp property in time
     series point can be absent. Known values are: "yearly", "monthly", "weekly", "daily",
     "hourly", "minutely", "secondly", "microsecond", and "none".
    :vartype granularity: str or ~anomalydetector.models.TimeGranularity
    :ivar custom_interval: Custom Interval is used to set non-standard time interval, for example,
     if the
     series is 5 minutes, request can be set as {"granularity":"minutely",
     "customInterval":5}.
    :vartype custom_interval: int
    :ivar period: Optional argument, periodic value of a time series. If the value is null or
     does not present, the API will determine the period automatically.
    :vartype period: int
    :ivar max_anomaly_ratio: Optional argument, advanced model parameter, max anomaly ratio in a
     time series.
    :vartype max_anomaly_ratio: float
    :ivar sensitivity: Optional argument, advanced model parameter, between 0-99, the lower the
     value
     is, the larger the margin value will be which means less anomalies will be
     accepted.
    :vartype sensitivity: int
    :ivar impute_mode: Used to specify how to deal with missing values in the input series, it's
     used
     when granularity is not "none". Known values are: "auto", "previous", "linear", "fixed",
     "zero", and "notFill".
    :vartype impute_mode: str or ~anomalydetector.models.ImputeMode
    :ivar impute_fixed_value: Used to specify the value to fill, it's used when granularity is not
     "none"
     and imputeMode is "fixed".
    :vartype impute_fixed_value: float
    """

    series: List["_models.TimeSeriesPoint"] = rest_field()
    """Time series data points. Points should be sorted by timestamp in ascending
order to match the anomaly detection result. If the data is not sorted
correctly or there is duplicated timestamp, the API will not work. In such
case, an error message will be returned. Required. """
    granularity: Optional[Union[str, "_models.TimeGranularity"]] = rest_field()
    """Optional argument, can be one of yearly, monthly, weekly, daily, hourly,
minutely, secondly, microsecond or none. If granularity is not present, it will
be none by default. If granularity is none, the timestamp property in time
series point can be absent. Known values are: \"yearly\", \"monthly\", \"weekly\", \"daily\", \"hourly\", \"minutely\", \"secondly\", \"microsecond\", and \"none\"."""
    custom_interval: Optional[int] = rest_field(name="customInterval")
    """Custom Interval is used to set non-standard time interval, for example, if the
series is 5 minutes, request can be set as {\"granularity\":\"minutely\",
\"customInterval\":5}. """
    period: Optional[int] = rest_field()
    """Optional argument, periodic value of a time series. If the value is null or
does not present, the API will determine the period automatically. """
    max_anomaly_ratio: Optional[float] = rest_field(name="maxAnomalyRatio")
    """Optional argument, advanced model parameter, max anomaly ratio in a time series. """
    sensitivity: Optional[int] = rest_field()
    """Optional argument, advanced model parameter, between 0-99, the lower the value
is, the larger the margin value will be which means less anomalies will be
accepted. """
    impute_mode: Optional[Union[str, "_models.ImputeMode"]] = rest_field(name="imputeMode")
    """Used to specify how to deal with missing values in the input series, it's used
when granularity is not \"none\". Known values are: \"auto\", \"previous\", \"linear\", \"fixed\", \"zero\", and \"notFill\"."""
    impute_fixed_value: Optional[float] = rest_field(name="imputeFixedValue")
    """Used to specify the value to fill, it's used when granularity is not \"none\"
and imputeMode is \"fixed\". """

    @overload
    def __init__(
        self,
        *,
        series: List["_models.TimeSeriesPoint"],
        granularity: Optional[Union[str, "_models.TimeGranularity"]] = None,
        custom_interval: Optional[int] = None,
        period: Optional[int] = None,
        max_anomaly_ratio: Optional[float] = None,
        sensitivity: Optional[int] = None,
        impute_mode: Optional[Union[str, "_models.ImputeMode"]] = None,
        impute_fixed_value: Optional[float] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class UnivariateEntireDetectionResult(_model_base.Model):
    """The response of entire anomaly detection.

    All required parameters must be populated in order to send to Azure.

    :ivar period: Frequency extracted from the series, zero means no recurrent pattern has been
     found. Required.
    :vartype period: int
    :ivar expected_values: ExpectedValues contain expected value for each input point. The index of
     the
     array is consistent with the input series. Required.
    :vartype expected_values: list[float]
    :ivar upper_margins: UpperMargins contain upper margin of each input point. UpperMargin is used
     to
     calculate upperBoundary, which equals to expectedValue + (100 -
     marginScale)*upperMargin. Anomalies in response can be filtered by
     upperBoundary and lowerBoundary. By adjusting marginScale value, less
     significant anomalies can be filtered in client side. The index of the array is
     consistent with the input series. Required.
    :vartype upper_margins: list[float]
    :ivar lower_margins: LowerMargins contain lower margin of each input point. LowerMargin is used
     to
     calculate lowerBoundary, which equals to expectedValue - (100 -
     marginScale)*lowerMargin. Points between the boundary can be marked as normal
     ones in client side. The index of the array is consistent with the input
     series. Required.
    :vartype lower_margins: list[float]
    :ivar is_anomaly: IsAnomaly contains anomaly properties for each input point. True means an
     anomaly either negative or positive has been detected. The index of the array
     is consistent with the input series. Required.
    :vartype is_anomaly: list[bool]
    :ivar is_negative_anomaly: IsNegativeAnomaly contains anomaly status in negative direction for
     each input
     point. True means a negative anomaly has been detected. A negative anomaly
     means the point is detected as an anomaly and its real value is smaller than
     the expected one. The index of the array is consistent with the input series. Required.
    :vartype is_negative_anomaly: list[bool]
    :ivar is_positive_anomaly: IsPositiveAnomaly contain anomaly status in positive direction for
     each input
     point. True means a positive anomaly has been detected. A positive anomaly
     means the point is detected as an anomaly and its real value is larger than the
     expected one. The index of the array is consistent with the input series. Required.
    :vartype is_positive_anomaly: list[bool]
    :ivar severity: The severity score for each input point. The larger the value is, the more
     sever the anomaly is. For normal points, the "severity" is always 0.
    :vartype severity: list[float]
    """

    period: int = rest_field()
    """Frequency extracted from the series, zero means no recurrent pattern has been
found. Required. """
    expected_values: List[float] = rest_field(name="expectedValues")
    """ExpectedValues contain expected value for each input point. The index of the
array is consistent with the input series. Required. """
    upper_margins: List[float] = rest_field(name="upperMargins")
    """UpperMargins contain upper margin of each input point. UpperMargin is used to
calculate upperBoundary, which equals to expectedValue + (100 -
marginScale)*upperMargin. Anomalies in response can be filtered by
upperBoundary and lowerBoundary. By adjusting marginScale value, less
significant anomalies can be filtered in client side. The index of the array is
consistent with the input series. Required. """
    lower_margins: List[float] = rest_field(name="lowerMargins")
    """LowerMargins contain lower margin of each input point. LowerMargin is used to
calculate lowerBoundary, which equals to expectedValue - (100 -
marginScale)*lowerMargin. Points between the boundary can be marked as normal
ones in client side. The index of the array is consistent with the input
series. Required. """
    is_anomaly: List[bool] = rest_field(name="isAnomaly")
    """IsAnomaly contains anomaly properties for each input point. True means an
anomaly either negative or positive has been detected. The index of the array
is consistent with the input series. Required. """
    is_negative_anomaly: List[bool] = rest_field(name="isNegativeAnomaly")
    """IsNegativeAnomaly contains anomaly status in negative direction for each input
point. True means a negative anomaly has been detected. A negative anomaly
means the point is detected as an anomaly and its real value is smaller than
the expected one. The index of the array is consistent with the input series. Required. """
    is_positive_anomaly: List[bool] = rest_field(name="isPositiveAnomaly")
    """IsPositiveAnomaly contain anomaly status in positive direction for each input
point. True means a positive anomaly has been detected. A positive anomaly
means the point is detected as an anomaly and its real value is larger than the
expected one. The index of the array is consistent with the input series. Required. """
    severity: Optional[List[float]] = rest_field()
    """The severity score for each input point. The larger the value is, the more
sever the anomaly is. For normal points, the \"severity\" is always 0. """

    @overload
    def __init__(
        self,
        *,
        period: int,
        expected_values: List[float],
        upper_margins: List[float],
        lower_margins: List[float],
        is_anomaly: List[bool],
        is_negative_anomaly: List[bool],
        is_positive_anomaly: List[bool],
        severity: Optional[List[float]] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class UnivariateLastDetectionResult(_model_base.Model):
    """The response of last anomaly detection.

    All required parameters must be populated in order to send to Azure.

    :ivar period: Frequency extracted from the series, zero means no recurrent pattern has been
     found. Required.
    :vartype period: int
    :ivar suggested_window: Suggested input series points needed for detecting the latest point.
     Required.
    :vartype suggested_window: int
    :ivar expected_value: Expected value of the latest point. Required.
    :vartype expected_value: float
    :ivar upper_margin: Upper margin of the latest point. UpperMargin is used to calculate
     upperBoundary, which equals to expectedValue + (100 - marginScale)*upperMargin.
     If the value of latest point is between upperBoundary and lowerBoundary, it
     should be treated as normal value. By adjusting marginScale value, anomaly
     status of latest point can be changed. Required.
    :vartype upper_margin: float
    :ivar lower_margin: Lower margin of the latest point. LowerMargin is used to calculate
     lowerBoundary, which equals to expectedValue - (100 - marginScale)*lowerMargin. Required.
    :vartype lower_margin: float
    :ivar is_anomaly: Anomaly status of the latest point, true means the latest point is an anomaly
     either in negative direction or positive direction. Required.
    :vartype is_anomaly: bool
    :ivar is_negative_anomaly: Anomaly status in negative direction of the latest point. True means
     the latest
     point is an anomaly and its real value is smaller than the expected one. Required.
    :vartype is_negative_anomaly: bool
    :ivar is_positive_anomaly: Anomaly status in positive direction of the latest point. True means
     the latest
     point is an anomaly and its real value is larger than the expected one. Required.
    :vartype is_positive_anomaly: bool
    :ivar severity: The severity score for the last input point. The larger the value is, the more
     sever the anomaly is. For normal points, the "severity" is always 0.
    :vartype severity: float
    """

    period: int = rest_field()
    """Frequency extracted from the series, zero means no recurrent pattern has been
found. Required. """
    suggested_window: int = rest_field(name="suggestedWindow")
    """Suggested input series points needed for detecting the latest point. Required. """
    expected_value: float = rest_field(name="expectedValue")
    """Expected value of the latest point. Required. """
    upper_margin: float = rest_field(name="upperMargin")
    """Upper margin of the latest point. UpperMargin is used to calculate
upperBoundary, which equals to expectedValue + (100 - marginScale)*upperMargin.
If the value of latest point is between upperBoundary and lowerBoundary, it
should be treated as normal value. By adjusting marginScale value, anomaly
status of latest point can be changed. Required. """
    lower_margin: float = rest_field(name="lowerMargin")
    """Lower margin of the latest point. LowerMargin is used to calculate
lowerBoundary, which equals to expectedValue - (100 - marginScale)*lowerMargin. Required. """
    is_anomaly: bool = rest_field(name="isAnomaly")
    """Anomaly status of the latest point, true means the latest point is an anomaly
either in negative direction or positive direction. Required. """
    is_negative_anomaly: bool = rest_field(name="isNegativeAnomaly")
    """Anomaly status in negative direction of the latest point. True means the latest
point is an anomaly and its real value is smaller than the expected one. Required. """
    is_positive_anomaly: bool = rest_field(name="isPositiveAnomaly")
    """Anomaly status in positive direction of the latest point. True means the latest
point is an anomaly and its real value is larger than the expected one. Required. """
    severity: Optional[float] = rest_field()
    """The severity score for the last input point. The larger the value is, the more
sever the anomaly is. For normal points, the \"severity\" is always 0. """

    @overload
    def __init__(
        self,
        *,
        period: int,
        suggested_window: int,
        expected_value: float,
        upper_margin: float,
        lower_margin: float,
        is_anomaly: bool,
        is_negative_anomaly: bool,
        is_positive_anomaly: bool,
        severity: Optional[float] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class VariableState(_model_base.Model):
    """Variable Status.

    :ivar variable: Variable name in variable states.
    :vartype variable: str
    :ivar filled_n_a_ratio: Proportion of missing values that need to be filled by fillNAMethod.
    :vartype filled_n_a_ratio: float
    :ivar effective_count: Number of effective data points before applying fillNAMethod.
    :vartype effective_count: int
    :ivar first_timestamp: First valid timestamp with value of input data.
    :vartype first_timestamp: ~datetime.datetime
    :ivar last_timestamp: Last valid timestamp with value of input data.
    :vartype last_timestamp: ~datetime.datetime
    """

    variable: Optional[str] = rest_field()
    """Variable name in variable states. """
    filled_n_a_ratio: Optional[float] = rest_field(name="filledNARatio")
    """Proportion of missing values that need to be filled by fillNAMethod. """
    effective_count: Optional[int] = rest_field(name="effectiveCount")
    """Number of effective data points before applying fillNAMethod. """
    first_timestamp: Optional[datetime.datetime] = rest_field(name="firstTimestamp")
    """First valid timestamp with value of input data. """
    last_timestamp: Optional[datetime.datetime] = rest_field(name="lastTimestamp")
    """Last valid timestamp with value of input data. """

    @overload
    def __init__(
        self,
        *,
        variable: Optional[str] = None,
        filled_n_a_ratio: Optional[float] = None,
        effective_count: Optional[int] = None,
        first_timestamp: Optional[datetime.datetime] = None,
        last_timestamp: Optional[datetime.datetime] = None,
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class VariableValues(_model_base.Model):
    """Variable values.

    All required parameters must be populated in order to send to Azure.

    :ivar variable: Variable name of last detection request. Required.
    :vartype variable: str
    :ivar timestamps: Timestamps of last detection request. Required.
    :vartype timestamps: list[str]
    :ivar values: Values of variables. Required.
    :vartype values: list[float]
    """

    variable: str = rest_field()
    """Variable name of last detection request. Required. """
    timestamps: List[str] = rest_field()
    """Timestamps of last detection request. Required. """
    values: List[float] = rest_field()
    """Values of variables. Required. """

    @overload
    def __init__(
        self,
        *,
        variable: str,
        timestamps: List[str],
        values: List[float],
    ):
        ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """
        ...

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, TYPE_CHECKING
from typing_extensions import Self

from azure.core.pipeline import policies
from azure.core.rest import HttpRequest, HttpResponse
from azure.mgmt.core import ARMPipelineClient
from azure.mgmt.core.policies import ARMAutoResourceProviderRegistrationPolicy

from . import models as _models
from ._configuration import DataFactoryManagementClientConfiguration
from ._serialization import Deserializer, Serializer
from .operations import (
    ActivityRunsOperations,
    ChangeDataCaptureOperations,
    CredentialOperationsOperations,
    DataFlowDebugSessionOperations,
    DataFlowsOperations,
    DatasetsOperations,
    ExposureControlOperations,
    FactoriesOperations,
    GlobalParametersOperations,
    IntegrationRuntimeNodesOperations,
    IntegrationRuntimeObjectMetadataOperations,
    IntegrationRuntimesOperations,
    LinkedServicesOperations,
    ManagedPrivateEndpointsOperations,
    ManagedVirtualNetworksOperations,
    Operations,
    PipelineRunsOperations,
    PipelinesOperations,
    PrivateEndPointConnectionsOperations,
    PrivateEndpointConnectionOperations,
    PrivateLinkResourcesOperations,
    TriggerRunsOperations,
    TriggersOperations,
)

if TYPE_CHECKING:
    from azure.core.credentials import TokenCredential


class DataFactoryManagementClient:  # pylint: disable=too-many-instance-attributes
    """The Azure Data Factory V2 management API provides a RESTful set of web services that interact
    with Azure Data Factory V2 services.

    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.datafactory.operations.Operations
    :ivar factories: FactoriesOperations operations
    :vartype factories: azure.mgmt.datafactory.operations.FactoriesOperations
    :ivar exposure_control: ExposureControlOperations operations
    :vartype exposure_control: azure.mgmt.datafactory.operations.ExposureControlOperations
    :ivar integration_runtimes: IntegrationRuntimesOperations operations
    :vartype integration_runtimes: azure.mgmt.datafactory.operations.IntegrationRuntimesOperations
    :ivar integration_runtime_object_metadata: IntegrationRuntimeObjectMetadataOperations
     operations
    :vartype integration_runtime_object_metadata:
     azure.mgmt.datafactory.operations.IntegrationRuntimeObjectMetadataOperations
    :ivar integration_runtime_nodes: IntegrationRuntimeNodesOperations operations
    :vartype integration_runtime_nodes:
     azure.mgmt.datafactory.operations.IntegrationRuntimeNodesOperations
    :ivar linked_services: LinkedServicesOperations operations
    :vartype linked_services: azure.mgmt.datafactory.operations.LinkedServicesOperations
    :ivar datasets: DatasetsOperations operations
    :vartype datasets: azure.mgmt.datafactory.operations.DatasetsOperations
    :ivar pipelines: PipelinesOperations operations
    :vartype pipelines: azure.mgmt.datafactory.operations.PipelinesOperations
    :ivar pipeline_runs: PipelineRunsOperations operations
    :vartype pipeline_runs: azure.mgmt.datafactory.operations.PipelineRunsOperations
    :ivar activity_runs: ActivityRunsOperations operations
    :vartype activity_runs: azure.mgmt.datafactory.operations.ActivityRunsOperations
    :ivar triggers: TriggersOperations operations
    :vartype triggers: azure.mgmt.datafactory.operations.TriggersOperations
    :ivar trigger_runs: TriggerRunsOperations operations
    :vartype trigger_runs: azure.mgmt.datafactory.operations.TriggerRunsOperations
    :ivar data_flows: DataFlowsOperations operations
    :vartype data_flows: azure.mgmt.datafactory.operations.DataFlowsOperations
    :ivar data_flow_debug_session: DataFlowDebugSessionOperations operations
    :vartype data_flow_debug_session:
     azure.mgmt.datafactory.operations.DataFlowDebugSessionOperations
    :ivar managed_virtual_networks: ManagedVirtualNetworksOperations operations
    :vartype managed_virtual_networks:
     azure.mgmt.datafactory.operations.ManagedVirtualNetworksOperations
    :ivar managed_private_endpoints: ManagedPrivateEndpointsOperations operations
    :vartype managed_private_endpoints:
     azure.mgmt.datafactory.operations.ManagedPrivateEndpointsOperations
    :ivar credential_operations: CredentialOperationsOperations operations
    :vartype credential_operations:
     azure.mgmt.datafactory.operations.CredentialOperationsOperations
    :ivar private_end_point_connections: PrivateEndPointConnectionsOperations operations
    :vartype private_end_point_connections:
     azure.mgmt.datafactory.operations.PrivateEndPointConnectionsOperations
    :ivar private_endpoint_connection: PrivateEndpointConnectionOperations operations
    :vartype private_endpoint_connection:
     azure.mgmt.datafactory.operations.PrivateEndpointConnectionOperations
    :ivar private_link_resources: PrivateLinkResourcesOperations operations
    :vartype private_link_resources:
     azure.mgmt.datafactory.operations.PrivateLinkResourcesOperations
    :ivar global_parameters: GlobalParametersOperations operations
    :vartype global_parameters: azure.mgmt.datafactory.operations.GlobalParametersOperations
    :ivar change_data_capture: ChangeDataCaptureOperations operations
    :vartype change_data_capture: azure.mgmt.datafactory.operations.ChangeDataCaptureOperations
    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials.TokenCredential
    :param subscription_id: The subscription identifier. Required.
    :type subscription_id: str
    :param base_url: Service URL. Default value is "https://management.azure.com".
    :type base_url: str
    :keyword api_version: Api Version. Default value is "2018-06-01". Note that overriding this
     default value may result in unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "TokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        self._config = DataFactoryManagementClientConfiguration(
            credential=credential, subscription_id=subscription_id, **kwargs
        )
        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                ARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: ARMPipelineClient = ARMPipelineClient(base_url=base_url, policies=_policies, **kwargs)

        client_models = {k: v for k, v in _models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.factories = FactoriesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.exposure_control = ExposureControlOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.integration_runtimes = IntegrationRuntimesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.integration_runtime_object_metadata = IntegrationRuntimeObjectMetadataOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.integration_runtime_nodes = IntegrationRuntimeNodesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.linked_services = LinkedServicesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.datasets = DatasetsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.pipelines = PipelinesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.pipeline_runs = PipelineRunsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.activity_runs = ActivityRunsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.triggers = TriggersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.trigger_runs = TriggerRunsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.data_flows = DataFlowsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.data_flow_debug_session = DataFlowDebugSessionOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.managed_virtual_networks = ManagedVirtualNetworksOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.managed_private_endpoints = ManagedPrivateEndpointsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.credential_operations = CredentialOperationsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.private_end_point_connections = PrivateEndPointConnectionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.private_endpoint_connection = PrivateEndpointConnectionOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.private_link_resources = PrivateLinkResourcesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.global_parameters = GlobalParametersOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.change_data_capture = ChangeDataCaptureOperations(
            self._client, self._config, self._serialize, self._deserialize
        )

    def _send_request(self, request: HttpRequest, *, stream: bool = False, **kwargs: Any) -> HttpResponse:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = client._send_request(request)
        <HttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.HttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> Self:
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details: Any) -> None:
        self._client.__exit__(*exc_details)



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# Below are common methods for the devops build steps. This is the common location that will be updated with
# package targeting during release.

import glob
import os
import errno
import sys
import logging

from subprocess import check_call, CalledProcessError, Popen
from argparse import Namespace
from typing import Iterable

# Assumes the presence of setuptools
from pkg_resources import parse_version, parse_requirements, Requirement, WorkingSet, working_set

# this assumes the presence of "packaging"
from packaging.specifiers import SpecifierSet

from ci_tools.functions import MANAGEMENT_PACKAGE_IDENTIFIERS, NO_TESTS_ALLOWED, lambda_filter_azure_pkg, str_to_bool
from ci_tools.parsing import parse_require, ParsedSetup

DEV_REQ_FILE = "dev_requirements.txt"
NEW_DEV_REQ_FILE = "new_dev_requirements.txt"

logging.getLogger().setLevel(logging.INFO)


def log_file(file_location, is_error=False):
    with open(file_location, "r") as file:
        for line in file:
            sys.stdout.write(line)
        sys.stdout.write("\n")
        sys.stdout.flush()


def read_file(file_location):
    str_buffer = ""
    with open(file_location, "r") as file:
        for line in file:
            str_buffer += line
    return str_buffer


def cleanup_folder(target_folder):
    for file in os.listdir(target_folder):
        file_path = os.path.join(target_folder, file)
        try:
            if os.path.isfile(file_path):
                os.remove(file_path)
        except Exception as e:
            logging.error(e)


# helper functions
def clean_coverage(coverage_dir):
    try:
        os.mkdir(coverage_dir)
    except OSError as e:
        if e.errno == errno.EEXIST:
            logging.info("Coverage dir already exists. Cleaning.")
            cleanup_folder(coverage_dir)
        else:
            raise


def run_check_call(
    command_array,
    working_directory,
    acceptable_return_codes=[],
    run_as_shell=False,
    always_exit=True,
):
    try:
        if run_as_shell:
            logging.info(
                "Command Array: {0}, Target Working Directory: {1}".format(" ".join(command_array), working_directory)
            )
            check_call(" ".join(command_array), cwd=working_directory, shell=True)
        else:
            logging.info("Command Array: {0}, Target Working Directory: {1}".format(command_array, working_directory))
            check_call(command_array, cwd=working_directory)
    except CalledProcessError as err:
        if err.returncode not in acceptable_return_codes:
            logging.error(err)  # , file = sys.stderr
            if always_exit:
                exit(1)
            else:
                return err


# This function generates code coverage parameters
def create_code_coverage_params(parsed_args: Namespace, package_path: str):
    coverage_args = []
    if parsed_args.disablecov:
        logging.info("Code coverage disabled as per the flag(--disablecov)")
        coverage_args.append("--no-cov")
    else:
        namespace = ParsedSetup.from_path(package_path).namespace
        coverage_args.append("--cov={}".format(namespace))
        coverage_args.append("--cov-append")
        logging.info(
            "Code coverage is enabled for package {0}, pytest arguements: {1}".format(
                namespace, coverage_args
            )
        )
    return coverage_args


# This function returns if error code 5 is allowed for a given package
def is_error_code_5_allowed(target_pkg, pkg_name):
    if (
        all(
            map(
                lambda x: any([pkg_id in x for pkg_id in MANAGEMENT_PACKAGE_IDENTIFIERS]),
                [target_pkg],
            )
        )
        or pkg_name in MANAGEMENT_PACKAGE_IDENTIFIERS
        or pkg_name in NO_TESTS_ALLOWED
    ):
        return True
    else:
        return False


# This method installs package from a pre-built whl
def install_package_from_whl(package_whl_path, working_dir, python_sym_link=sys.executable):
    commands = [
        python_sym_link,
        "-m",
        "pip",
        "install",
        package_whl_path,
        "--extra-index-url",
        "https://pypi.python.org/simple",
    ]
    run_check_call(commands, working_dir)
    logging.info("Installed package from {}".format(package_whl_path))


def filter_dev_requirements(pkg_root_path, packages_to_exclude, dest_dir):
    # This method returns list of requirements from dev_requirements by filtering out packages in given list
    dev_req_path = os.path.join(pkg_root_path, DEV_REQ_FILE)
    if not os.path.exists(dev_req_path):
        logging.info("{0} is not found in package root {1}".format(DEV_REQ_FILE, pkg_root_path))
        return ""

    requirements = []
    with open(dev_req_path, "r") as dev_req_file:
        requirements = dev_req_file.readlines()

    # filter any package given in excluded list
    requirements = [req for req in requirements if os.path.basename(req.replace("\n", "")) not in packages_to_exclude]

    logging.info("Filtered dev requirements: {}".format(requirements))
    # create new dev requirements file with different name for filtered requirements
    new_dev_req_path = os.path.join(dest_dir, NEW_DEV_REQ_FILE)
    with open(new_dev_req_path, "w") as dev_req_file:
        dev_req_file.writelines(requirements)

    return new_dev_req_path


def extend_dev_requirements(dev_req_path, packages_to_include):
    requirements = []
    with open(dev_req_path, "r") as dev_req_file:
        requirements = dev_req_file.readlines()

    # include any package given in included list. omit duplicate
    for requirement in packages_to_include:
        if requirement not in requirements:
            requirements.insert(0, requirement.rstrip() + "\n")

    logging.info("Extending dev requirements. New result:: {}".format(requirements))
    # create new dev requirements file with different name for filtered requirements
    with open(dev_req_path, "w") as dev_req_file:
        dev_req_file.writelines(requirements)


def is_required_version_on_pypi(package_name: str, spec: str) -> bool:
    """
    This function evaluates a package name and version specifier combination and returns the versions on pypi
    that satisfy the provided version specifier.

    Import dependency on azure-sdk-tools.
    """

    from pypi_tools.pypi import PyPIClient

    client = PyPIClient()
    versions = []
    try:
        versions = client.get_ordered_versions(package_name)

        if spec:
            specifier = SpecifierSet(spec)
            versions = [str(v) for v in versions if v in specifier]
    except:
        logging.error("Package {} is not found on PyPI".format(package_name))
    return bool(versions)


def find_packages_missing_on_pypi(path: str) -> Iterable[str]:
    """
    Given a setup path, evaluate all dependencies and return a list of packages whos specifier can NOT be matched against PyPI releases.

    Import dependency on pkginfo.
    """

    import pkginfo

    requires = []
    if path.endswith(".whl"):
        requires = list(filter(lambda_filter_azure_pkg, pkginfo.get_metadata(path).requires_dist))
    else:
        requires = ParsedSetup.from_path(path).requires

    # parse pkg name and spec
    pkg_spec_dict = [parse_require(req) for req in requires]
    logging.info("Package requirement: {}".format(pkg_spec_dict))
    # find if version is available on pypi
    missing_packages = [
        f"{pkg.key}{pkg.specifier}"
        for pkg in pkg_spec_dict
        if not is_required_version_on_pypi(pkg.key, str(pkg.specifier))
    ]
    if missing_packages:
        logging.error("Packages not found on PyPI: {}".format(missing_packages))
    return missing_packages


def find_tools_packages(root_path):
    """Find packages in tools directory. For e.g. azure-sdk-tools"""
    glob_string = os.path.join(root_path, "tools", "*", "setup.py")
    pkgs = [os.path.basename(os.path.dirname(p)) for p in glob.glob(glob_string)]
    logging.info("Packages in tools: {}".format(pkgs))
    return pkgs


def get_installed_packages(paths=None):
    """Find packages in default or given lib paths"""
    # WorkingSet returns installed packages in given path
    # working_set returns installed packages in default path
    # if paths is set then find installed packages from given paths
    ws = WorkingSet(paths) if paths else working_set
    return ["{0}=={1}".format(p.project_name, p.version) for p in ws]




# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See LICENSE.txt in the project root for
# license information.
# -------------------------------------------------------------------------
from azure.cosmos.aio import CosmosClient
import azure.cosmos.exceptions as exceptions
from azure.cosmos.partition_key import PartitionKey
import azure.cosmos.documents as documents

import asyncio
import config
import json
from typing import Dict, Any

# ----------------------------------------------------------------------------------------------------------
# Prerequisites -
#
# 1. An Azure Cosmos account -
#    https://learn.microsoft.com/azure/cosmos-db/create-sql-api-python#create-a-database-account
#
# 2. Microsoft Azure Cosmos
#    pip install azure-cosmos>=4.0.0
# ----------------------------------------------------------------------------------------------------------
# Sample - how to get and use resource token that allows restricted access to data
# ----------------------------------------------------------------------------------------------------------
# Note:
#
# This sample creates a Container to your database account.
# Each time a Container is created the account will be billed for 1 hour of usage based on
# the provisioned throughput (RU/s) of that account.
# ----------------------------------------------------------------------------------------------------------

HOST = config.settings["host"]
MASTER_KEY = config.settings["master_key"]
DATABASE_ID = config.settings["database_id"]
CONTAINER_ID = config.settings["container_id"]
PARTITION_KEY = PartitionKey(path="/username")

# User that you want to give access to
USERNAME, USERNAME_2 = "user", "user2"

CONTAINER_ALL_PERMISSION = "CONTAINER_ALL_PERMISSION"
PARTITION_READ_PERMISSION = "PARTITION_READ_PERMISSION"
DOCUMENT_ALL_PERMISSION = "DOCUMENT_ALL_PERMISSION"


async def create_user_if_not_exists(db, username):
    try:
        user = await db.create_user(body={"id": username})
    except exceptions.CosmosResourceExistsError:
        user = db.get_user_client(username)

    return user


async def create_permission_if_not_exists(user, permission_definition):
    try:
        permission = await user.create_permission(permission_definition)
    except exceptions.CosmosResourceExistsError:
        permission = await user.read_permission(permission_definition["id"])

    return permission


async def token_client_upsert(container, username, item_id):
    try:
        await container.upsert_item(
            {
                "id": item_id,
                "username": username,
                "msg": "This is a message for " + username,
            }
        )
    except exceptions.CosmosHttpResponseError:
        print("Error in upserting item with id '{0}'.".format(item_id))


async def token_client_read_all(container):
    try:
        items = container.read_all_items()
        async for i in items:
            print(i)
    except exceptions.CosmosResourceNotFoundError:
        print("Cannot read items--container '{0}' not found.".format(container.id))
    except exceptions.CosmosHttpResponseError:
        print("Error in reading items in container '{0}'.".format(container.id))


async def token_client_read_item(container, username, item_id):
    try:
        item = await container.read_item(item=item_id, partition_key=username)
        print(item)
    except exceptions.CosmosResourceNotFoundError:
        print("Cannot read--item with id '{0}' not found.".format(item_id))
    except exceptions.CosmosHttpResponseError:
        print("Error in reading item with id '{0}'.".format(item_id))


async def token_client_delete(container, username, item_id):
    try:
        await container.delete_item(item=item_id, partition_key=username)
    except exceptions.CosmosResourceNotFoundError:
        print("Cannot delete--item with id '{0}' not found.".format(item_id))
    except exceptions.CosmosHttpResponseError:
        print("Error in deleting item with id '{0}'.".format(item_id))


async def token_client_query(container, username):
    try:
        async for item in container.query_items(
            query="SELECT * FROM my_container c WHERE c.username=@username",
            parameters=[{"name": "@username", "value": username}],
            partition_key=username,
        ):
            print(json.dumps(item, indent=True))
    except exceptions.CosmosHttpResponseError:
        print("Error in querying item(s)")


async def run_sample():
    async with CosmosClient(HOST, MASTER_KEY) as client:

        try:
            try:
                db = await client.create_database(DATABASE_ID)
            except exceptions.CosmosResourceExistsError:
                db = client.get_database_client(DATABASE_ID)

            try:
                container = await db.create_container(
                    id=CONTAINER_ID, partition_key=PARTITION_KEY
                )
            except exceptions.CosmosResourceExistsError:
                container = db.get_container_client(CONTAINER_ID)

            user = await create_user_if_not_exists(db, USERNAME)

            # Permission to perform operations on all items inside a container
            permission_definition: Dict[str, Any] = {
                "id": CONTAINER_ALL_PERMISSION,
                "permissionMode": documents.PermissionMode.All,
                "resource": container.container_link,
            }

            permission = await create_permission_if_not_exists(user, permission_definition)
            token = {}
            token[container.container_link] = permission.properties["_token"]

            # Use token to connect to database
            # If you initialize the asynchronous client without using 'async with' in your context,
            # make sure to close the client once you're done using it
            token_client = CosmosClient(HOST, token)
            token_db = token_client.get_database_client(DATABASE_ID)
            token_container = token_db.get_container_client(CONTAINER_ID)

            ITEM_1_ID, ITEM_2_ID, ITEM_3_ID = "1", "2", "3"

            # Update or insert item if not exists
            await token_client_upsert(token_container, USERNAME, ITEM_1_ID)
            await token_client_upsert(token_container, USERNAME, ITEM_2_ID)
            await token_client_upsert(token_container, USERNAME_2, ITEM_3_ID)

            # Read all items in the container, across all partitions
            await token_client_read_all(token_container)

            # Read specific item
            await token_client_read_item(token_container, USERNAME, ITEM_2_ID)

            # Query for items in a certain partition
            await token_client_query(token_container, USERNAME_2)

            # Delete an item
            await token_client_delete(token_container, USERNAME, ITEM_2_ID)

            # Give user read-only permission, for a specific partition
            user_2 = await create_user_if_not_exists(db, USERNAME_2)
            permission_definition = {
                "id": PARTITION_READ_PERMISSION,
                "permissionMode": documents.PermissionMode.Read,
                "resource": container.container_link,
                "resourcePartitionKey": [USERNAME_2],
            }
            permission = await create_permission_if_not_exists(user_2, permission_definition)
            read_token = {}
            read_token[container.container_link] = permission.properties["_token"]

            # Closing current token client in order to re-initialize with read_token below:
            await token_client.close()

            # Use token to connect to database
            # If you initialize the asynchronous client without using 'async with' make sure to close it once you're done
            token_client = CosmosClient(HOST, read_token)
            token_db = token_client.get_database_client(DATABASE_ID)
            token_container = token_db.get_container_client(CONTAINER_ID)

            # Fails since this client has access to only items with partition key USERNAME_2 (ie. "user2")
            await token_client_read_all(token_container)

            # Ok to read item(s) with partition key "user2"
            await token_client_read_item(token_container, USERNAME_2, ITEM_3_ID)

            # Can't upsert or delete since it's read-only
            await token_client_upsert(token_container, USERNAME_2, ITEM_3_ID)

            # Give user CRUD permissions, only for a specific item
            item_3 = await token_container.read_item(item=ITEM_3_ID, partition_key=USERNAME_2)
            permission_list = user_2.list_permissions()
            async for p in permission_list:
                await user_2.delete_permission(p.get('id'))
            user_2_permissions = [permission async for permission in user_2.list_permissions()]
            assert len(user_2_permissions) == 0

            permission_definition = {
                "id": DOCUMENT_ALL_PERMISSION,
                "permissionMode": documents.PermissionMode.All,
                "resource": str(item_3.get('_self')) #this identifies the item with id "3"
            }

            permission = await create_permission_if_not_exists(user_2, permission_definition)

            item_token = {}
            item_token[container.container_link] = permission.properties["_token"]

            # Closing current token client in order to re-initialize with item_token below:
            await token_client.close()

            # Use token to connect to database
            token_client = CosmosClient(HOST, item_token)
            token_db = token_client.get_database_client(DATABASE_ID)
            token_container = token_db.get_container_client(CONTAINER_ID)

            # Fails since this client only has access to a specific item
            await token_client_read_all(token_container)

            # Fails too, for same reason
            await token_client_read_item(token_container, USERNAME, ITEM_1_ID)

            # Ok to perform operations on that specific item
            await token_client_read_item(token_container, USERNAME_2, ITEM_3_ID)
            await token_client_delete(token_container, USERNAME_2, ITEM_3_ID)

            # Cleaning up and closing current token client
            await token_client.delete_database(DATABASE_ID)
            await token_client.close()  

        except exceptions.CosmosHttpResponseError as e:
            print("\nrun_sample has caught an error. {0}".format(e.message))

        finally:
            print("\nrun_sample done")


if __name__ == "__main__":
    asyncio.run(run_sample())



# ------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# -------------------------------------------------------------------------

from azure.core import MatchConditions
from azure.core.exceptions import (
    ResourceExistsError,
    ResourceModifiedError,
    ResourceNotModifiedError)


def prep_if_match(etag, match_condition):
    # type: (str, MatchConditions) -> Optional[str]
    error_map = {}
    if match_condition == MatchConditions.IfNotModified:
        if not etag:
            raise ValueError("The 'IfNotModified' match condition must be paired with an etag.")
        error_map[412] = ResourceModifiedError
        return etag, error_map
    if match_condition == MatchConditions.IfPresent:
        if etag:
            raise ValueError("An etag value cannot be paired with the 'IfPresent' match condition.")
        return "*", error_map
    if match_condition != MatchConditions.Unconditionally:
        raise ValueError("Unsupported match condition: {}".format(match_condition))
    return None, error_map

def prep_if_none_match(etag, match_condition):
    # type: (str, MatchConditions) -> Optional[str]
    error_map = {}
    if match_condition == MatchConditions.IfModified:
        error_map[412] = ResourceNotModifiedError
        if not etag:
            raise ValueError("The 'IfModified' match condition must be paired with an etag.")
        return etag, error_map
    if match_condition == MatchConditions.IfMissing:
        error_map[412] = ResourceExistsError
        if etag:
            raise ValueError("An etag value cannot be paired with the 'IfMissing' match condition.")
        return "*", error_map
    if match_condition != MatchConditions.Unconditionally:
        raise ValueError("Unsupported match condition: {}".format(match_condition))
    return None, error_map



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: sample_classify_document.py

DESCRIPTION:
    This sample demonstrates how to classify a document using a trained document classifier.
    To learn how to build your custom classifier, see sample_build_classifier.py.

    More details on building a classifier and labeling your data can be found here:
    https://aka.ms/azsdk/documentintelligence/buildclassifiermodel

USAGE:
    python sample_classify_document.py

    Set the environment variables with your own values before running the sample:
    1) DOCUMENTINTELLIGENCE_ENDPOINT - the endpoint to your Document Intelligence resource.
    2) DOCUMENTINTELLIGENCE_API_KEY - your Document Intelligence API key.
    3) CLASSIFIER_ID - the ID of your trained document classifier
        -OR-
       DOCUMENTINTELLIGENCE_TRAINING_DATA_CLASSIFIER_SAS_URL - The shared access signature (SAS) Url of your Azure Blob Storage container
       with your training files. A document classifier will be built and used to run the sample.
"""

import os


def classify_document(classifier_id):
    path_to_sample_documents = os.path.abspath(
        os.path.join(
            os.path.abspath(__file__),
            "..",
            "./sample_forms/forms/IRS-1040.pdf",
        )
    )

    # [START classify_document]
    from azure.core.credentials import AzureKeyCredential
    from azure.ai.documentintelligence import DocumentIntelligenceClient
    from azure.ai.documentintelligence.models import AnalyzeResult

    endpoint = os.environ["DOCUMENTINTELLIGENCE_ENDPOINT"]
    key = os.environ["DOCUMENTINTELLIGENCE_API_KEY"]
    classifier_id = os.getenv("CLASSIFIER_ID", classifier_id)

    document_intelligence_client = DocumentIntelligenceClient(endpoint=endpoint, credential=AzureKeyCredential(key))
    with open(path_to_sample_documents, "rb") as f:
        poller = document_intelligence_client.begin_classify_document(classifier_id, body=f)
    result: AnalyzeResult = poller.result()

    print("----Classified documents----")
    if result.documents:
        for doc in result.documents:
            if doc.bounding_regions:
                print(
                    f"Found document of type '{doc.doc_type or 'N/A'}' with a confidence of {doc.confidence} contained on "
                    f"the following pages: {[region.page_number for region in doc.bounding_regions]}"
                )
    # [END classify_document]


if __name__ == "__main__":
    import uuid
    from azure.core.exceptions import HttpResponseError
    from dotenv import find_dotenv, load_dotenv

    try:
        load_dotenv(find_dotenv())
        classifier_id = None
        if os.getenv("DOCUMENTINTELLIGENCE_TRAINING_DATA_CLASSIFIER_SAS_URL") and not os.getenv("CLASSIFIER_ID"):
            from azure.core.credentials import AzureKeyCredential
            from azure.ai.documentintelligence import DocumentIntelligenceAdministrationClient
            from azure.ai.documentintelligence.models import (
                AzureBlobContentSource,
                ClassifierDocumentTypeDetails,
                BuildDocumentClassifierRequest,
            )

            endpoint = os.environ["DOCUMENTINTELLIGENCE_ENDPOINT"]
            key = os.environ["DOCUMENTINTELLIGENCE_API_KEY"]
            blob_container_sas_url = os.environ["DOCUMENTINTELLIGENCE_TRAINING_DATA_CLASSIFIER_SAS_URL"]

            document_intelligence_admin_client = DocumentIntelligenceAdministrationClient(
                endpoint=endpoint, credential=AzureKeyCredential(key)
            )

            poller = document_intelligence_admin_client.begin_build_classifier(
                BuildDocumentClassifierRequest(
                    classifier_id=str(uuid.uuid4()),
                    doc_types={
                        "IRS-1040-A": ClassifierDocumentTypeDetails(
                            azure_blob_source=AzureBlobContentSource(
                                container_url=blob_container_sas_url, prefix="IRS-1040-A/train"
                            )
                        ),
                        "IRS-1040-B": ClassifierDocumentTypeDetails(
                            azure_blob_source=AzureBlobContentSource(
                                container_url=blob_container_sas_url, prefix="IRS-1040-B/train"
                            )
                        ),
                    },
                )
            )
            classifier = poller.result()
            classifier_id = classifier.classifier_id
        classify_document(classifier_id)
    except HttpResponseError as error:
        # Examples of how to check an HttpResponseError
        # Check by error code:
        if error.error is not None:
            if error.error.code == "InvalidImage":
                print(f"Received an invalid image error: {error.error}")
            if error.error.code == "InvalidRequest":
                print(f"Received an invalid request error: {error.error}")
            # Raise the error again after printing it
            raise
        # If the inner error is None and then it is possible to check the message to get more information:
        if "Invalid request".casefold() in error.message.casefold():
            print(f"Uh-oh! Seems there was an invalid request: {error}")
        # Raise the error again
        raise



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import sys
import glob
import os
import argparse
from collections import Counter
from subprocess import check_call, CalledProcessError

root_dir = os.path.abspath(os.path.join(os.path.abspath(__file__), "..", ".."))


def pip_command(command, additional_dir=".", error_ok=False):
    try:
        print("Executing: {} from {}".format(command, additional_dir))
        check_call(
            [sys.executable, "-m", "pip"] + command.split(),
            cwd=os.path.join(root_dir, additional_dir),
        )
        print()
    except CalledProcessError as err:
        print(err, file=sys.stderr)
        if not error_ok:
            sys.exit(1)

def select_install_type(pkg, run_develop, exceptions):
    # the default for disable_develop will be false, which means `run_develop` will be true
    argument = ""
    if run_develop:
        argument = "-e"

    if pkg in exceptions:
        # opposite of whatever our decision was
        if argument == "":
            argument = "-e"
        elif argument == "-e":
            argument = ""

    return argument

# optional argument in a situation where we want to build a variable subset of packages
parser = argparse.ArgumentParser(
    description="Set up the dev environment for selected packages."
)
parser.add_argument(
    "--packageList",
    "-p",
    dest="packageList",
    default="",
    help="Comma separated list of targeted packages. Used to limit the number of packages that dependencies will be installed for.",
)
parser.add_argument(
    "--disabledevelop",
    dest="install_in_develop_mode",
    default=True,
    action="store_false",
    help="Add this argument if you would prefer to install the package with a simple `pip install` versus `pip install -e`",
)
# this is a hack to support generating docs for the single package that doesn't support develop mode. It will be removed when we
# migrate to generating docs on a per-package cadence.
parser.add_argument(
    "--exceptionlist",
    "-e",
    dest="exception_list",
    default="",
    help="Comma separated list of packages that we want to take the 'opposite' installation method for.",
)

args = parser.parse_args()

packages = {
    tuple(os.path.dirname(f).rsplit(os.sep, 1))
    for f in glob.glob(os.path.join(root_dir, "sdk/*/azure-*/setup.py")) + glob.glob(os.path.join(root_dir, "tools/azure-*/setup.py"))
}
# [(base_folder, package_name), ...] to {package_name: base_folder, ...}
packages = {package_name: base_folder for (base_folder, package_name) in packages}

exceptions = [p.strip() for p in args.exception_list.split(',')]

# keep targeted packages separate. python2 needs the nspkgs to work properly.
if not args.packageList:
    targeted_packages = list(packages.keys())
else:
    targeted_packages = [
        os.path.relpath(x.strip()) for x in args.packageList.split(",")
    ]

# Extract nspkg and sort nspkg by number of "-"
nspkg_packages = [p for p in packages.keys() if "nspkg" in p]
nspkg_packages.sort(key=lambda x: len([c for c in x if c == "-"]))

# Meta-packages to ignore
meta_packages = ["azure-keyvault", "azure-mgmt", "azure"]

content_packages = sorted(
    [
        p
        for p in packages.keys()
        if p not in nspkg_packages + meta_packages and p in targeted_packages
    ]
)

# Install tests dep first
if "azure-sdk-tools" in content_packages:
    content_packages.remove("azure-sdk-tools")
content_packages.insert(1, "azure-sdk-tools")

# Put azure-common in front of content package
if "azure-common" in content_packages:
    content_packages.remove("azure-common")
content_packages.insert(2, "azure-common")

if 'azure-core' in content_packages:
    content_packages.remove('azure-core')
content_packages.insert(3, 'azure-core')


print("Running dev setup...")
print("Root directory '{}'\n".format(root_dir))

# install private whls if there are any
privates_dir = os.path.join(root_dir, "privates")
if os.path.isdir(privates_dir) and os.listdir(privates_dir):
    whl_list = " ".join(
        [os.path.join(privates_dir, f) for f in os.listdir(privates_dir)]
    )
    pip_command("install {}".format(whl_list))

# install nspkg only on py2, but in wheel mode (not editable mode)
if sys.version_info < (3,):
    for package_name in nspkg_packages:
        pip_command("install {}/{}/".format(packages[package_name], package_name))



# install packages
print("Packages to install: {}".format(content_packages))
for package_name in content_packages:
    print("\nInstalling {}".format(package_name))
    # if we are running dev_setup with no arguments. going after dev_requirements will be a pointless exercise
    # and waste of cycles as all the dependencies will be installed regardless.
    if os.path.isfile(
        "{}/{}/dev_requirements.txt".format(packages[package_name], package_name)
    ):
        pip_command(
            "install -r dev_requirements.txt",
            os.path.join(packages[package_name], package_name),
        )

    pip_command(
        "install --ignore-requires-python {} {}".format(
            select_install_type(package_name, args.install_in_develop_mode, exceptions),
            os.path.join(packages[package_name], package_name)
        )
    )

# On Python 3, uninstall azure-nspkg if he got installed
if sys.version_info >= (3,):
    pip_command("uninstall -y azure-nspkg", error_ok=True)

print("Finished dev setup.")

